// -----------------------------------------------------------------------
// COMPONENT - IPTV
// -----------------------------------------------------------------------
Vue.component('iptv', {
  props: {
    area: {
      type: Object
    },
    parent: {
      type: Object
    },
    device_id: {
      type: String
    },
    factor_width: {
      type: Number
    },
    factor_height: {
      type: Number
    }
  },
  components: {
  },
  data: function () {
    return {
      muted: false
    }
  },
  created: function () {
    var self = this
  },
  mounted: function () {
    var self = this
    self.setup()
  },
  computed: {
  },
  methods: {
    setup: function () {
      var self = this
      // console.log('iptv', self.area.config)
      if (self.parent.isTizen() && _.includes(self.parent.tvFirmwareVersion, '2.4')) {
        self.debug('IPTV not supported on Tizen 2.4')
      } else {
        self.startPlaying()
      }
    },
    hideElemByClass: function (className) {
      var elements = document. getElementsByClassName(className)
      _.forEach(elements, function (el) {
        el.style.display = 'none'
      })
    },
    startPlaying: function () {
      var self = this

      var videoSrc = self.area.config.iptv_url

      if (!videoSrc) {
        self.parent.debug('No IPTV url defined...')
        return
      }

      if (self.area.config.play_sound == 0) {
        self.muted = true
      } else {
        self.muted = false
      }

      var player = videojs('my-video', { controls: false })
      player.muted(self.muted)

      player.on('error', function (e) {
        console.error('Video.js Error:', e)
      })

      player.src({
        src: videoSrc,
        type: 'application/x-mpegURL',
      })

      var myDiv = document.getElementById('my-div')
      var elementsInsideDiv = myDiv.querySelectorAll('*')
      var didHide = {}
      _.forEach(elementsInsideDiv, function (el, idx) {
        // if (el.id && !didHide.hasOwnProperty(el.id)) {
        //  didHide[el.id] = 1
        if (el.id != 'my-video_html5_api' && el.id != 'my-video' && el.id != 'my-div') {
          // console.log('id', el.id)
          el.style.display = 'none'
        }
        // }
      })
    },
    debug: function (msg) {
      var self = this
      console.log('debug', msg)
      self.$emit('debug', 'iptv - ' + msg)
    },
  },
  watch: {
    'parent.playlistRestart': function (res) {
      var self = this
      self.setup()
    }
  },
  template: '<div _style="background-color: red;" id="my-div">' +
    '<video id="my-video" poster="images/video-poster.png" autoplay style="width: 100%; height: 100%;"></video>' +
    '</div>'
})
// -----------------------------------------------------------------------
// END IPTV
// -----------------------------------------------------------------------
