// -----------------------------------------------------------------------
// COMPONENT - MENU-BOARD
// -----------------------------------------------------------------------
Vue.component('menu-board', {
  props: {
    area: {
      type: Object
    },
    parent: {
      type: Object
    },
    device_id: {
      type: String
    },
    factor_width: {
      type: Number
    },
    factor_height: {
      type: Number
    }
  },
  data: function () {
    return {
      timerConnectError: undefined,
      ws: undefined,
      firstGroup: '',
      menuConfig: [],
      started: false,
      pages: [],
      playlistMap: [],
      pageCounter: 0,
      timeoutNextPage: 5000,
      fontFactorGroup: 1,
      fontFactorProduct: 1,
      lineSpacing: 0,
      show: false,
      transactionInEfect: 'animated fadeInLeftBig',
      transactionOutEfect: 'animated fadeOutLeftBig',
      effects: {
        fadeBig: { in: 'fadeInLeftBig', out: 'fadeOutLeftBig' },
        fade: { in: 'fadeIn', out: 'fadeOut' },
        bounce: { in: 'bounceIn', out: 'bounceOut' },
        lightSpeed: { in: 'lightSpeedInRight', out: 'lightSpeedOutLeft' },
        zoom: { in: 'zoomIn', out: 'zoomOut' },
        slide: { in: 'slideInDown', out: 'slideOutDown' },
        back: { in: 'backInLeft', out: 'backOutRight' }
      },
      pageInterval: undefined,
      pageIntervalTimeout: undefined
    }
  },
  created: function () {
    var self = this
    // alert('text')
    self.checkAnimation()
  },
  mounted: function () {
    var self = this
    self.setup()
  },
  computed: {
    // width: function () {
    //   var self = this
    //   return (self.area.width * self.factor_width) + 'px'
    // },
    // height: function () {
    //   var self = this
    //   return (self.area.height * self.factor_height) + 'px'
    // }
  },
  watch: {
    watch: {
      'parent.playlistRestart': function (res) {
        var self = this
        self.setup()
      }
    },
    'parent.wsConnected': function (res) {
      var self = this
      // console.log('menu-board - wsConnected')
      self.parent.wsSend({
        origin: 'browser',
        listen_ip: self.parent.kioskIp,
        websocket_token: self.parent.kioskToken,
        device_id: self.device_id,
        configKey: self.area.configKey,
        areaKey: self.area.areaKey,
        action: 'menu_config_request',
        params: null
      })
    },
    'parent.wsMessage': function (res) {
      var self = this
      if (res.origin != 'linux') return
      if (res.action) {
        switch (res.action) {
          case 'menu_config':
            // console.log('Got menu_config from ws')
            // make sure we get the correct kiosk_config
            clearTimeout(self.timerConnectError)
            self.menuConfig = res
            // console.log('menu_config', self.menuConfig)
            if (res.info.hasOwnProperty(self.area.configKey) && res.info[self.area.configKey].hasOwnProperty(self.area.areaKey)) {
              self.menuConfig = res.info[self.area.configKey][self.area.areaKey]
              // console.log('menuConfig', self.menuConfig)
              if (self.menuConfig.groups.length == 0) {
                self.debug('Families not defined in ' + self.area.areaKey + ' and ' + self.area.configKey + ' on Menu Editor')
              }
              if (!self.started) {
                self.startPlay()
              }
            }
              // console.log('menu_config', self.menu_config)
            break
          case 'menu_changed':
            self.started = false
            self.pageCounter = 0
            self.parent.wsSend({
              origin: 'browser',
              listen_ip: self.parent.kioskIp,
              websocket_token: self.parent.kioskToken,
              device_id: self.device_id,
              configKey: self.area.configKey,
              areaKey: self.area.areaKey,
              action: 'menu_config_request',
              params: null
            })
            break
          default:
            // console.log('invalid action - ', res.action)
        }
      }
    }
  },
  methods: {
    setup: function () {
      var self = this
      // console.log('menu-board mounted', self.area)
      if (self.area.config.font_size_group) self.fontFactorGroup = self.area.config.font_size_group / 10
      if (self.area.config.font_size_product) self.fontFactorProduct = self.area.config.font_size_product / 10
      if (self.area.config.line_spacing) self.lineSpacing = self.area.config.line_spacing
      self.checkChangeDay()
    },
    checkAnimation: function () {
      var self = this
      // console.log('checkAnimation', self.area.config.effect)
      if (!self.area.config.effect) {
        self.transactionInEfect = 'animated fadeIn'
        self.transactionOutEfect = 'animated fadeOut'
      } else {
        self.transactionInEfect = 'animated ' + self.effects[self.area.config.effect].in
        self.transactionOutEfect = 'animated ' + self.effects[self.area.config.effect].out
      }
      // console.log('transactionInEfect', self.transactionInEfect)
      // console.log('transactionOutEfect', self.transactionOutEfect)
    },

    genStyleGroups: function (index) {
      var self = this
      var style = 'font-size:' + (2 * self.fontFactorGroup) + 'vh; color:' + self.area.config.color_group + '; font-weight:' + (self.area.config.bold_text_group == 1 ? 'bold' : '') + '; font-style:' + (self.area.config.italic_text_group == 1 ? 'italic' : '') + '; padding-top:' + (index == 0 ? '1vh' : '2vh')
      // console.log('style', style)
      if (index != self.firstGroup) {
        style = 'margin-top: ' + self.lineSpacing + 'vh; ' + style
      }
      return style
    },

    genStyleProducts: function (menuproductindex, menuproduct) {
      var self = this
      var style = 'margin-top: ' + self.lineSpacing + 'vh; ' + 'font-size:' + (2 * self.fontFactorProduct) + 'vh; color:' + self.area.config.color_product + '; padding-top:' + (menuproductindex == 0 ? '1vh' : '1vh') + '; font-weight:' + (self.area.config.bold_text_product == 1 ? 'bold' : '') + '; font-style:' + (self.area.config.italic_text_product == 1 ? 'italic' : '') + '; text-decoration:' + (menuproduct.unavailable == 1 ? 'line-through' : '') + ';' + (self.area.config.products_border ? 'border-bottom: 1px ' + self.area.config.products_border + ' ' + 'black;' : '')
      // console.log('styleProducts', style)
      return style
    },

    checkChangeDay: function () {
      var self = this
      // time until midnight of the current day
      var now = new Date(self.parent.curDate)
      var timeToMidnight = new Date(
        now.getFullYear(),
        now.getMonth(),
        now.getDate() + 1,
        0,
        0,
        0
      ) - now
      // console.log('timeToMidnight', timeToMidnight)
      // Timeout for first execution at midnight
      setTimeout(function () {
        self.startPlay()
      }, timeToMidnight)

      // Starts the interval to run the function at midnight every day
      self.parent.createInterval(function () {
        self.startPlay()
      }, 24 * 60 * 60 * 1000) // 24h interval

    },

    startPageInterval: function () {
      var self = this
      if (!self.pageInterval) {
        // console.log('startPageInterval')
        self.pageInterval = self.parent.createInterval(function () {
          if (self.pageCounter >= self.pages.length - 1) {
            self.pageCounter = 0
            return
          }
          self.pageCounter = self.pageCounter + 1
        }, self.timeoutNextPage * 1000)
        self.pageIntervalTimeout = self.timeoutNextPage
      } else {
        if (self.pageIntervalTimeout != self.timeoutNextPage) {
          self.parent.deleteInterval(self.pageInterval)
          self.pageInterval = self.parent.createInterval(function () {
            if (self.pageCounter >= self.pages.length - 1) {
              self.pageCounter = 0
              return
            }
            self.pageCounter = self.pageCounter + 1
          }, self.timeoutNextPage * 1000)
          self.pageIntervalTimeout = self.timeoutNextPage
        }
      }
    },

    startPlay: function () {
      var self = this
      self.started = true
      if (self.menuConfig.page_change_delay) self.timeoutNextPage = self.menuConfig.page_change_delay * 1
      self.playlistMap = _.groupBy(self.menuConfig.groups, 'name')

      var nowDay = new Date(self.parent.curDate).getDay() + 1 // one more because 0 is Sunday and on Menu Editor 1 is Sunday
      _.forEach(self.playlistMap, function (fam, idx) { // loop by family
        if (self.firstGroup == '') self.firstGroup = idx
        var filterProduts = _.filter(fam[0].products, function (prod) { // filter products by current day
          if (prod.week_days.length == 0) return true
          return _.includes(prod.week_days, nowDay) && prod.off != 1
        })
        fam[0].products = filterProduts // replace the array of products with the filtered array corresponding to the current day of the week
      })

      _.forEach(self.menuConfig.groups, function (group, gindex) {
        _.forEach(group.page, function (gp, gpindex) {
          self.pages.push(gp)
        })
      })
      self.pages = _.uniq(self.pages)
      self.startPageInterval()
      // console.log('self.playlistMap', self.playlistMap)
      // console.log('pages', self.pages)
    },

    startConnectErrorTimer: function () {
      var self = this
      clearTimeout(self.timerConnectError)
      self.timerConnectError = setTimeout(function () {
        location.reload()
      }, 20000)
    },
    debug: function (msg, force) {
      var self = this
      console.log(msg)
      self.$emit('debug', 'menu-board ' + msg)
    }
  },
  template:
' <div>' +
'   <div v-for="(page, pgindex) in pages"> ' +
'     <div v-if="pages[pageCounter] == page"> ' +
// '  <transition class="group" appear enter-active-class="animated fadeIn" leave-active-class="animated fadeOut">' +
'       <div v-for="(menu, menuindex) in playlistMap" v-if="menu[0].page.includes(page)"> ' +
'         <div class="row" :style="genStyleGroups(menuindex)">' +
'           <div class="col" style="padding-left: 1.5vh;">{{ menu[0].name }}</div>' +
'           <div v-if="menu[0].price1_title" class="col-2 text-right text-no-wrap" style="padding-right: 1.5vh;">{{ menu[0].price1_title }}</div>' +
'           <div v-if="menu[0].price2_title" class="col-2 text-right text-no-wrap" style="padding-right: 1.5vh;">{{ menu[0].price2_title }}</div>' +
'           <div v-if="menu[0].price3_title" class="col-2 text-right text-no-wrap" style="padding-right: 1.5vh;">{{ menu[0].price3_title }}</div>' +
'         </div> ' +
'         <div v-for="(menuproduct, menuproductindex) in menu[0].products">' +
'           <div class="row" :style="genStyleProducts(menuproductindex, menuproduct)">' +
'             <div class="col" style="padding-left: 1.5vh;"> {{ menuproduct.name }} </div>' +
'             <div v-if="menuproduct.price1" class="col-2 text-right text-no-wrap" style="padding-right: 1.5vh;">{{ menuproduct.price1 }}</div>' +
'             <div v-if="menuproduct.price2" class="col-2 text-right text-no-wrap" style="padding-right: 1.5vh;">{{ menuproduct.price2 }}</div>' +
'             <div v-if="menuproduct.price3" class="col-2 text-right text-no-wrap" style="padding-right: 1.5vh;">{{ menuproduct.price3 }}</div>' +
'           </div>' +
'         </div>' +
'       </div> ' +
// '  </transition> ' +
'     </div> ' +
'    </div> ' +
' </div> '
})
// -----------------------------------------------------------------------
// END MENU-BOARD
// -----------------------------------------------------------------------
