// -----------------------------------------------------------------------
// COMPONENT - QUEUES
// -----------------------------------------------------------------------
Vue.component('queues', {
  props: {
    area: {
      type: Object
    },
    parent: {
      type: Object
    },
    device_id: {
      type: String
    },
    factor_width: {
      type: Number
    },
    factor_height: {
      type: Number
    },
    kiosk_ip: {
      type: String
    },
    kiosk_token: {
      type: String
    }
  },
  data: function () {
    return {
      ws: undefined,
      wsInterval: undefined,
      kiosk_config: undefined,
      queues: [],
      ticket_called: undefined,
      queueNames: {},
      fontFactor: 1,
      hiddeLetter: false,
      calledTickets: [],
      timerConnectError: undefined,
      timer: undefined,
      ticketPrinting: false,
      queuesToUse: {},
      lastCalledTickets: [],
      timeoutHideCall1: undefined,
      timeoutHideCall2: undefined,
      pendingTickets: {},
      queuesLastNumberCalled: {},
      // colorQueuesCached: undefined,
      borderRadius: undefined,
      myhtml: '',
      ticket_recall: undefined,
      firstTicketsInfo: true,
      inYellow: false,
      show: true,
      lastQueueCalled: undefined,
      previousBgColor: undefined,
      previousColor: undefined,
      firstConfigRequest: true
    }
  },
  created: function () {
    var self = this
    // make sure kiosk_token is correct
    if (self.kiosk_token && self.kiosk_token == 'undefined') self.kiosk_token = ''
  },
  mounted: function () {
    var self = this
    self.setup()
  },
  watch: {
    'parent.playlistRestart': function (res) {
      var self = this
      self.setup()
    },
    'parent.wsConnected': function (res) {
      var self = this
      if (res) {
        var msg = {
          origin: 'browser',
          listen_ip: self.parent.kioskIp,
          websocket_token: self.parent.kioskToken,
          device_id: self.device_id,
          configKey: self.area.configKey,
          areaKey: self.area.areaKey,
          action: 'kiosk_config_request',
          params: null
        }
        if (self.firstConfigRequest) {
          self.firstConfigRequest = false
          console.log('wsConnected, will send kiosk_config_request!!!')
          self.parent.wsSend(msg)
        } else {
          setTimeout(function () {
            console.log('wsConnected, will send kiosk_config_request!!!')
            self.parent.wsSend(msg)
          }, 5000)
        }
      }
    },
    'parent.wsMessage': function (res) {
      var self = this
      if (res.origin != 'linux') return
      if (res.action) {
        // console.log('res', res)
        switch (res.action) {
          case 'kiosk_config':
            console.log('got kiosk_config!!!')
            // make sure we get the correct kiosk_config
            clearTimeout(self.timerConnectError)
            self.kiosk_config = res
            var sortedKeys = _.sortBy(_.keys(self.kiosk_config.info.queues))
            self.queues = []
            self.queueNames = {}
            _.forEach(sortedKeys, function (queue) {
              var obj = self.kiosk_config.info.queues[queue]
              self.queues.push(obj)
            })
            _.forEach(self.queues, function (el) {
              self.queueNames[el.queue] = el.name
            })
            break
          case 'tickets_info':
            // console.log('tickets_info!!!')
            if (res.info.hasOwnProperty('last_called_tickets')) {
              if (res.info.last_called_tickets.length > 0) {
                res.info.last_called_tickets = _.sortBy(res.info.last_called_tickets, function (obj) { obj.row_number })
                if (res.info.last_called_tickets.length > self.area.config.show_last_called_number) {
                  if (self.area.config.show_actual_call * 1 == 1) {
                    self.lastCalledTickets = res.info.last_called_tickets.slice(0, self.area.config.show_last_called_number)
                  } else {
                    var num = self.area.config.show_last_called_number * 1 + 1
                    self.lastCalledTickets = res.info.last_called_tickets.slice(0, num)
                  }
                } else {
                  self.lastCalledTickets = res.info.last_called_tickets
                }
              }
            }
            self.pendingTickets = {}
            if (res.info.hasOwnProperty('pending_tickets')) {
              _.forEach(res.info.pending_tickets, function (el, idx) {
                self.$set(self.pendingTickets, el.queue, el.pending_tickets)
              })
            }
            break
          case 'tickets_create':
            // console.log('tickets_create!!')
            break
          case 'screens_restart':
            location.reload()
            break
          case 'ticket_call_repeat':
            // console.log('ticket_call_repeat', res.ticket)
            if (Object.keys(self.queuesToUse).length > 0) {
              if (!self.queuesToUse.hasOwnProperty(res.ticket.queue)) return
            }
            if (res.ticket.recall * 1 == 1) {
              self.ticket_recall = res.ticket
              self.ticket_called = undefined
            } else {
              self.ticket_called = res.ticket
              self.ticket_recall = undefined
            }
            self.doTicketCall()
            break
          case 'ticket_called':

            if (Object.keys(self.queuesToUse).length > 0) {
              if (!self.queuesToUse.hasOwnProperty(res.ticket.queue)) return
            }

            // on 'call' areas, don't react to recalls
            if (self.area.config.type == 'screen' && res.ticket.recall * 1 == 1) {
              return
            }

            if (res.ticket.recall * 1 == 1) {
              // console.log('ticket_called1')
              self.ticket_recall = res.ticket
              self.ticket_called = undefined
            } else {
              // console.log('ticket_called2')
              self.ticket_called = res.ticket
              self.ticket_recall = undefined
              self.queues = self.queues
            }

            self.doTicketCall()

            break
          default:
            // console.log('invalid action - ', res.action)
        }
      }
    }
  },
  computed: {
    lastTicketsToShow: function () {
      var self = this
      var tickets = []
      if (Object.keys(self.queuesToUse).length > 0) {
        tickets = _.filter(self.lastCalledTickets, function (el) {
          return self.queuesToUse.hasOwnProperty(el.queue) ? true : false
        })
      } else {
        tickets = self.lastCalledTickets
      }
      if (self.area.config.show_last_called * 1 == 1 && self.area.config.show_actual_call * 1 == 1) {
        return tickets
      } else {
        const tickets = _.slice(tickets, 1, tickets.length)
        return tickets
      }
    },
    queues_with_called: function () {
      var self = this
      var arr = []
      // console.log('queues', self.queues)
      _.forEach(self.queues, function (el, idx) {
        if (self.ticket_called && self.ticket_called.queue == el.queue) {
          el.called = true
        } else {
          el.called = false
        }
        _.forEach(self.kiosk_config.info.queues, function (value, key) {
          if (value.visible == '1') {
            if (key == el.queue) {
              el.inactive = '0'
            }
          } else {
            if (key == el.queue) {
              el.inactive = '1'
            }
          }
        })

        if (el.last_number_called == '--' && self.queuesLastNumberCalled.hasOwnProperty(el.queue)) {
          el.last_number_called = self.setFormatQueues(self.queuesLastNumberCalled[el.queue], 1)
        } else {
          el.last_number_called = self.setFormatQueues(el.last_number_called, 2)
        }

        arr.push(el)
      })
      if (Object.keys(self.queuesToUse).length > 0) {
        self.queues = _.filter(self.queues, function (el) {
          return self.queuesToUse.hasOwnProperty(el.queue) ? true : false
        })
      }
      return self.queues
    }
  },
  methods: {
    queueClick: function (idx, queue) {
      var self = this
      // console.log('queueClick', idx, queue)
      if (self.ticketPrinting) return
      if (self.area.config.to_create_tickets * 1 == 0) return
      if (!self.kiosk_ip) return

      var url = self.parent.http_method + self.kiosk_ip + '/ticket_create'
      var xhr = new XMLHttpRequest()
      xhr.onreadystatechange = function () {
        if (xhr.readyState == XMLHttpRequest.DONE) {
          if (xhr.status == 200) {
            var ref = JSON.parse(xhr.responseText)

            if (ref.hasOwnProperty('success')) {
              var toPrint = 1

              if (self.kiosk_config.hasOwnProperty('info') && self.kiosk_config.info.queues.hasOwnProperty(queue.queue)) {
                toPrint = self.kiosk_config.info.queues[queue.queue].print * 1
              }

              if (toPrint == 1) {
                self.printTicket(ref.content)
              }
            }
          } else {
            // error
          }
        }
      }
      xhr.open('POST', url)
      var data = new FormData()
      data.append('queue', queue.queue)
      xhr.send(data)
    },
    printTicket: function (content) {
      var self = this

      self.ticketPrinting = true

      setTimeout(function () {
        self.ticketPrinting = false
      }, 3000)

      var url = self.parent.http_method + self.kiosk_ip + '/print'
      var xhr = new XMLHttpRequest()
      xhr.onreadystatechange = function () {
        if (xhr.readyState == XMLHttpRequest.DONE) {
          if (xhr.status == 200) {
            // console.log('xhr.responseText', xhr.responseText);
            // var ref = JSON.parse(xhr.responseText)
            // if (ref.hasOwnProperty('success')) {
            // }
          } else {
            // error
          }
        }
      }
      xhr.open('POST', url)
      var data = new FormData()
      data.append('print', content)
      xhr.send(data)

    },
    blink: function () {
      var self = this
      if (self.area.config.blink_on_call * 1 == 0) return

      var inc = 150 // speed
      var num = 15  // number of times

      var newInc = inc
      self.show = false

      for (var i=0; i<num; i++) {
        setTimeout(function () {
          self.show = !self.show
          if (i == num - 1) self.show = true
        }, newInc)
        newInc = newInc + inc
      }
    },
    setup: function () {
      var self = this
      // console.log('AREA', self.area.config)
      if (self.area.config.queues_to_use) {
        var arr = _.split(self.area.config.queues_to_use, ',')
        _.forEach(arr, function (q) {
          self.queuesToUse[q] = 1
        })
      }
      if (self.area.config.font_size) self.fontFactor = self.area.config.font_size / 10
      if (self.area.config.border_radius) self.borderRadius = self.area.config.border_radius

      setTimeout(function () {
        // console.log('kiosk_config', self.kiosk_config, self.kiosk_config.info.show_queue_letter * 1)
        if (self.kiosk_config) {
          if (self.kiosk_config.info.show_queue_letter * 1 == 1) {
            self.hiddeLetter = false
          } else {
            self.hiddeLetter = true
          }
          self.setColorQueues()
        }
      }, 2000)

      // if (self.area.config.show_last_called * 1 == 0 || !self.area.config.hasOwnProperty('keep_lastcalled_yellow')) {
      if (!self.area.config.hasOwnProperty('keep_lastcalled_yellow')) {
        self.area.config.keep_lastcalled_yellow = 0
      }
    },
    debug: function (msg, force) {
      var self = this
      console.log(msg)
      self.$emit('debug', 'queues - ' + msg)
    },
    startConnectErrorTimer: function () {
      var self = this
      clearTimeout(self.timerConnectError)
      self.timerConnectError = setTimeout(function () {
        location.reload()
      }, 15000)
    },
    doTicketCall: function () {
      var self = this
      var ticket = self.ticket_recall != undefined ? self.ticket_recall : self.ticket_called
      if (!ticket) return

      self.lastQueueCalled = ticket.queue

      if (ticket.origin_number && ticket.origin_number == '0') {
        ticket.origin_number = ''
      }

      // console.log('ticket', ticket)

      if (self.area.config.type == 'call') {
        if (self.area.config.recall * 1 == 1 && ticket.recall * 1 != 1) {
          return
        } else if (self.area.config.priority * 1 == 1 && ticket.priority * 1 != 1) {
          return
        }
        if (self.area.config.accumulates_calls * 1 == 1) {
          self.addToArray(ticket)
        } else {
          self.calledTickets = []
          if (self.calledTickets.length == 0) {
            self.addToArray(ticket)
          }
        }
      } else {
        if (ticket.recall * 1 == 1) return
        if (self.area.config.show_last_called * 1 == 1) {
          self.addToLastCalledTickets(ticket)
        }
      }

      clearTimeout(self.timeoutHideCall1)
      if (self.area.config.show_actual_call * 1 == 1) {
        // console.log('self.inYellow = true - 1')
        console.log('addToLastCalledTickets2')
        self.inYellow = true
      }
      if (self.area.config.keep_lastcalled_yellow * 1 == 0) {
        self.timeoutHideCall1 = setTimeout(function () {
          var div = document.getElementById('identify-call-' + self.area.areaKey + '-' + self.lastQueueCalled)
          if (div) {
            div.style.backgroundColor = self.previousBgColor
            div.style.color = self.previousColor
          }
          self.lastQueueCalled = undefined
          self.inYellow = false
          self.ticket_called = undefined
          self.ticket_recall = undefined
          ticket = undefined
        }, 10000)
      }

      var originNumber = ticket.origin_number || ticket.number
      var originQueue = ticket.origin_queue || ticket.queue
      self.$set(self.queuesLastNumberCalled, originQueue, originNumber)

      // console.log('self.area.config.type', self.area.config.type)
      // console.log('ticket.recall', ticket.recall)

      /* if (self.area.config.type == 'call' && ticket.recall * 1 == 1) {
        // when doing a recall, don't update the queues...
      } else {
        _.forEach(self.queues, function (el, idx) {
          if (el.queue == ticket.queue) {
            el.last_station_who_called = ticket.last_station_who_called
            el.last_number_called = originNumber
            el.last_queue_called = originQueue
            return false
          }
        })
      } */

      // lets update the info in the queue related with the ticket that was called
      _.forEach(self.queues, function (el, idx) {
        if (el.queue == ticket.queue) {
          el.last_station_who_called = ticket.last_station_who_called
          el.last_number_called = originNumber
          el.last_queue_called = originQueue
          return false
        }
      })

      if (self.area.config.play_sound && ticket.sound) {
        var queue = ticket.origin_queue || ticket.queue
        var number = ticket.origin_number || ticket.number
        if (ticket.sound == '1') ticket.sound = 'dingdong'
        ticket.sound = ticket.sound.replace('QUEUE_KEY', queue)
        ticket.sound = ticket.sound.replace('NUMBER', number)
        ticket.sound = ticket.sound.replace('STATION', ticket.station)
        var origSound = ticket.sound
        if (ticket.lang && ticket.lang.toUpperCase() != 'PT') {
          ticket.sound = ticket.sound.replace(new RegExp('^MAN/', 'g'), 'MAN/' + ticket.lang.toUpperCase() + '/')
          ticket.sound = ticket.sound.replace(new RegExp(',MAN/', 'g'), ',MAN/' + ticket.lang.toUpperCase() + '/')

          ticket.sound = ticket.sound.replace(new RegExp('^WOMAN/', 'g'), 'WOMAN/' + ticket.lang.toUpperCase() + '/')
          ticket.sound = ticket.sound.replace(new RegExp(',WOMAN/', 'g'), ',WOMAN/' + ticket.lang.toUpperCase() + '/')
          var bilingue = ticket.bilingue || 'false'
          if (String(bilingue) == 'true') {
            ticket.sound = ticket.sound + ',' + origSound
          }
        }
        var soundsArray = ticket.sound.split(',')
        if (soundsArray.length > 0) {
          for (var i=0; i<soundsArray.length; i++) {
            self.parent.playSounds(soundsArray[i], self.kiosk_ip)
          }
        }
      }

      self.blink()
    },
    setColorQueues: function () {
      var self = this
      // if (self.colorQueuesCached) return self.colorQueuesCached
      var colorQueue = {}
      _.forEach(self.kiosk_config.info.queues, function (value, key) {
        if (self.area.config.type == 'call') {
          colorQueue[key] = {
            background_color: self.area.config.background_color ? self.area.config.background_color : value.color,
            color: self.area.config.foreground_color ? self.area.config.foreground_color : 'black'
          }
        } else {
          colorQueue[key] = {
            background_color: value.color ? value.color : 'white',
            color: self.area.config.foreground_color ? self.area.config.foreground_color : 'black'
          }
        }
      })
      // self.colorQueuesCached = colorQueue
      return colorQueue
    },
    getColorQueue: function (queue) {
      var self = this
      var colors = self.setColorQueues()
      var colorObj = {
        background_color: '#FFFFFF',
        color: '#000000'
      }
      if (!queue) return colorObj
      if (colors.hasOwnProperty(queue)) {
        var bgColor = colors[queue].background_color
        if (bgColor) colorObj.background_color = bgColor

        var color = colors[queue].color
        if (color) colorObj.color = color
      }
      return colorObj
    },
    setFormatQueues: function (lastCalledNumber, idxDebug) {
      var self = this
      // console.log('setFormatQueues', idxDebug, objDebug)
      // if (! lastCalledNumber) return
      if (!self.kiosk_config.info.queue_numbers_format) {
        self.kiosk_config.info.queue_numbers_format = '00'
      }
      if (self.kiosk_config.info.queue_numbers_format.length == 2) {
        if (lastCalledNumber && lastCalledNumber.length == 1) {
          return "0" + lastCalledNumber
        } else {
          return lastCalledNumber
        }
      } else if (self.kiosk_config.info.queue_numbers_format.length == 3) {
        if (lastCalledNumber && lastCalledNumber.length == 1) {
          return "00" + lastCalledNumber
        } else if (lastCalledNumber && lastCalledNumber.length == 2) {
          return "0" + lastCalledNumber
        } else {
          return lastCalledNumber
        }
      } else if (self.kiosk_config.info.queue_numbers_format.length == 4) {
        if (lastCalledNumber && lastCalledNumber.length == 1) {
          return "000" + lastCalledNumber
        } else if (lastCalledNumber && lastCalledNumber.length == 2) {
          return "00" + lastCalledNumber
        } else if (lastCalledNumber && lastCalledNumber.length == 3) {
          return "0" + lastCalledNumber
        } else {
          return lastCalledNumber
        }
      } else {
        return lastCalledNumber
      }
    },
    addToArray: function(ticket) {
      var self = this
      // self.calledTickets.push(ticket)
      if (Object.keys(self.queuesToUse).length > 0) {
        if (!self.queuesToUse.hasOwnProperty(ticket.queue)) return
      }
      self.calledTickets.unshift(ticket)
      if (!self.timer) {
        self.startTimer()
      }
    },
    startTimer: function() {
      var self = this
      var timeout = self.area.config.call_timeout * 1000
      self.timer = setTimeout(function () {
        if (self.area.config.call_timeout == '' || self.area.config.call_timeout == undefined) {
          if (self.calledTickets.length > 1) {
            self.calledTickets.pop()
          } else {
            return
          }
        } else {
          self.calledTickets.pop()
        }
        if (self.calledTickets.length > 0) {
          self.startTimer()
        } else {
          self.timer = null
        }
      }, timeout)
    },
    formatTicketToText: function (ticket) {
      var self = this
      var txt = self.area.config.text_call
      var queue = ticket.origin_queue || ticket.queue
      var number = ticket.origin_number || ticket.number || ticket.last_number_called
      if (self.hiddeLetter == true) {
        txt = txt.replace('#queue#', '')
      } else {
        txt = txt.replace('#queue#', queue)
      }
      txt = txt.replace('#name#', self.queueNames.hasOwnProperty(queue) ? self.queueNames[queue] : ticket.name)
      txt = txt.replace('#number#', self.setFormatQueues(number, 3))
      txt = txt.replace('#station#', ticket.station)
      txt = txt.replace('#station_name#', ticket.station_name)
      txt = txt.replace('#color#', ticket.color)
      txt = txt.replace('#customer_name#', ticket.customer_name)
      txt = txt.replace('#aux_info1#', ticket.aux_info1)
      txt = txt.replace('#aux_info2#', ticket.aux_info2)
      return txt
    },
    addToLastCalledTickets: function (ticket) {
      var self = this
      console.log('addToLastCalledTickets')
      if (self.area.config.show_last_called * 1 == 1 && self.area.config.show_actual_call * 1 == 0) {
        self.area.config.show_last_called_number = self.area.config.show_last_called_number * 1 + 1
      }
      if (self.area.config.show_last_called * 1 == 1 && self.lastCalledTickets.length < self.area.config.show_last_called_number) {
        self.lastCalledTickets.unshift(ticket)
      } else {
        self.lastCalledTickets.pop()
        self.lastCalledTickets.unshift(ticket)
      }

      console.log('addToLastCalledTickets1')

      // console.log('aqui2', self.area.config.show_last_called)
      clearTimeout(self.timeoutHideCall2)
      if (self.area.config.show_actual_call * 1 == 1) {
        // console.log('self.inYellow = true - 2')
        self.inYellow = true
      }
      if (self.area.config.keep_lastcalled_yellow * 1 == 0) {
        self.timeoutHideCall2 = setTimeout(function () {
          var div = document.getElementById('identify-call-' + self.area.areaKey + '-' + self.lastQueueCalled)
          if (div) {
            div.style.backgroundColor = self.previousBgColor
            div.style.color = self.previousColor
          }
          self.inYellow = false
          self.lastQueueCalled = undefined
          self.ticket_called = undefined
          ticket.called = false
        }, 10000)
      }

      if (self.area.config.show_last_called * 1 == 1 && self.area.config.show_actual_call * 1 == 0) {
        self.area.config.show_last_called_number = self.area.config.show_last_called_number * 1 - 1
      }
    },
    customQueues: function (queue, showlastcall) {
      var self = this
      var lastQueue = queue.last_queue_called || queue.queue
      var html = ''

      html = self.area.config.input_html
      html = html.replace(/#areakey_queue#/gi, self.area.areaKey + '-' + lastQueue)

      if (showlastcall == 'showlastcall') {
        html = html.replace(/#queue_name#/gi, queue.name)
        html = html.replace(/#queue_station#/gi, queue.station || queue.last_station_who_called)
        html = html.replace(/#bg_station_color#/gi, self.getColorQueue(queue.queue).background_color)
        html = html.replace(/#queue_number#/gi, self.setFormatQueues(queue.number || queue.last_number_called, 4))

        if (self.hiddeLetter) {
          html = html.replace(/#queue_queue#/gi, '')
        } else {
          html = html.replace(/#queue_queue#/gi, queue.queue)
        }
      } else {
        html = html.replace(/#queue_name#/gi, queue.name)
        html = html.replace(/#queue_station#/gi, queue.last_station_who_called || queue.station)
        html = html.replace(/#bg_station_color#/gi, self.getColorQueue(queue.queue).background_color)
        if (self.hiddeLetter) {
          html = html.replace(/#queue_queue#/gi, '')
        } else {
          html = html.replace(/#queue_queue#/gi, lastQueue)
        }

        if (self.ticket_called && lastQueue == self.ticket_called.queue) {
          html = html.replace(/#queue_number#/gi, self.setFormatQueues(self.ticket_called.number || self.ticket_called.last_number_called, 5))
        } else {
          html = html.replace(/#queue_number#/gi, queue.last_number_called)
        }
      }

      /* console.log('self.area.areaKey', self.area.areaKey)
      console.log('self.lastQueueCalled', self.lastQueueCalled)
      console.log('queue.called', queue.called)
      console.log('self.inYellow', self.inYellow) */

      setTimeout(function () {
        var div = document.getElementById('identify-call-' + self.area.areaKey + '-' + self.lastQueueCalled)
        // console.log('div', div)
        if (div) {
          if ((queue.hasOwnProperty('virtual_kiosk') && queue.virtual_kiosk == '1') || (queue.called && self.area.config.keep_lastcalled_yellow == 1) || self.inYellow) {
            if (div.style.backgroundColor != 'yellow') {
              self.previousBgColor = div.style.backgroundColor
              self.previousColor = div.style.color
            }
            if (self.area.config.show_last_called * 1 == 1 && self.area.config.show_actual_call * 1 == 0) {
            } else {
              div.style.backgroundColor = 'yellow'
              div.style.color = 'black'
            }
          }
        }
      }, 50)

      return html
    }
  },
  template: '<div v-show="show">' +
    '<div v-if="!kiosk_ip">' +
      '<center><h3 style="margin-top: 150px; background-color: white; color: black;">NO KIOSK IP DEFINED</h3></center>' +
    '</div>' +
    '<div class="absolute-center" v-else-if="!kiosk_config || !queues.length">' +
    '  <q-spinner-ios color="primary" style="font-size: 13vh;"/>' +
    '</div>' +
    '<div v-else-if="area.config.queue_layout == 4 && area.config.type != \'call\' && area.config.show_last_called != 1">' +
      '<!-- pre>{{ queues_with_called }}</pre --><center>' +
      '<q-spinner-ios v-if="ticketPrinting" color="primary" :style="\'font-size: \' + (9 * fontFactor) + \'vh; padding-top: \' + (3 * fontFactor) + \'vh;\'"/>' +
      '<table v-else border=0 width=100%>' +
        ' <tr v-if="queues_with_called.length > 0 && area.config.word_ticket && area.config.word_post" style="border-collapse: collapse; ">' +
          '  <th nowrap :style="\'padding:\' + (1 * fontFactor) + \'vh; font-size:\' + (1.5 * fontFactor) + \'vh; font-weight: bold;\'"><i style="font-size: 1em;" class="fas fa-receipt"></i> {{ area.config.word_ticket || \'SENHA\' }}</th>' +
          '  <th nowrap :style="\'padding:\' + (1 * fontFactor) + \'vh; font-size:\' + (1.5 * fontFactor) + \'vh; font-weight: bold;\'"> {{ area.config.word_post || \'POSTO\' }} <i style="font-size: 1em;" class="fas fa-concierge-bell"></i></th>' +
          '</tr>' +
        '<template v-for="queue in queues_with_called">' +
          ' <tr :key="queue" v-if="queue.inactive != \'1\'" @click="queueClick(1, queue)">' +
          '   <td width="65%" :id="\'queue-color\' + \'-\' + queue.last_queue_called ? queue.last_queue_called : queue.queue" :style="\'font-weight: bolder; text-align: center; padding: 2vh 0 2vh 0; background-color: \' + (queue.called && area.config.show_actual_call * 1 == 1 ? \'yellow\' : \'white\') + \'; border-radius: 3vh 0 0 3vh; box-shadow: 0px 2px 4px 0px rgba(0,0,0,0.5); font-size:\' + (5 * fontFactor) + \'vh; color: black;\'">{{ hiddeLetter ? \'\' : queue.last_queue_called ? queue.last_queue_called : queue.queue }}{{ ticket_called && (ticket_called.origin_queue || ticket_called.queue) == queue.last_queue_called ? setFormatQueues(ticket_called.origin_number || ticket_called.number || ticket_called.last_number_called, 6) : setFormatQueues(queue.last_number_called, 7) }}</td>' +
          '   <td width="35%" :style="\'font-weight: bolder; text-align: center; padding: 2vh 0 2vh 0; background-color: \' + getColorQueue(queue.last_queue_called ? queue.last_queue_called : queue.queue).background_color + \'; border-radius: 0 3vh 3vh 0; box-shadow: 0px 2px 4px 0px rgba(0,0,0,0.5); font-size:\' + (5 * fontFactor) + \'vh; color: black;\'">{{ queue.last_station_who_called }}</td>' +
          ' </tr>' +
          '<tr></tr><tr></tr><tr></tr><tr></tr>' +
        '</template>' +
      '</table></center>' +
    '</div>' +
    '<div v-else-if="area.config.queue_layout == 5 && area.config.type != \'call\' && queues.length">' +
      '<center>' +
      '<table border=0 cellspacing=0 width="100%">' +
      '<template v-for="(q, key) in queues_with_called">' +
      '<tr :key="key" @click="queueClick(2, q)">'+
        '<td width="100%" :style="\'font-weight: bolder; text-align: left; padding: 2vh; background-color: \' + (q.called && area.config.show_actual_call * 1 == 1 ? \'yellow\' : \'white\') + \'; border-radius: 2vh; box-shadow: 2px 2px 2px 2px rgba(174,174,174,0.6); font-size: \' + (1.5 * fontFactor) + \'vh; color: black;\'">' +
          '<q-spinner-ios v-if="ticketPrinting" color="primary" :style="\'font-size: \' + (1.5 * fontFactor) + \'vh;\'"/>' +
          '<table v-else width=100% border=0 cellpadding=0 cellspacing=0>' +
          '  <tr>' +
          '    <td v-if="kiosk_config && kiosk_config.info.show_queue_letter * 1 == 1" nowrap valign="top" width=10%>{{ q.queue }} -</td>' +
          '    <td valign=top width=90%>' +
          '      <table width=100% border=0 cellpadding=0 cellspacing=0>' +
          '        <tr><td>{{ q.name }}</td></tr>' +
          '        <tr><td v-if="area.config.show_pending_tickets" style="font-size: 0.5em;" align=right>Em espera - {{ pendingTickets[q.queue] || 0 }}</td></tr>' +
          '      </table>' +
          '    </td>' +
          '  </tr>' +
          '</table>' +
        '</td>' +
      '</tr>' +
      '<tr><td></td></tr><tr><td></td></tr><tr><td></td></tr>' +
      '</template>' +
      '</table>' +
      '</center>' +
    '</div>' +
    '<div v-else-if="area.config.queue_layout != 5 && area.config.queue_layout != 6 && area.config.type != \'call\' && area.config.show_last_called == 1">' +
      '<center><table border=0 _cellpadding=0 _cellspacing=10 width=100%>' +
        ' <tr v-if="lastTicketsToShow.length > 0 && area.config.word_ticket && area.config.word_post" style="border-collapse: collapse; ">' +
          '  <th nowrap :style="\'padding:\' + (1 * fontFactor) + \'vh; font-size:\' + (1.5 * fontFactor) + \'vh; font-weight: bold;\'"><i style="font-size: 1em;" class="fas fa-receipt"></i> {{ area.config.word_ticket || \'SENHA\' }}</th>' +
          '  <th nowrap :style="\'padding:\' + (1 * fontFactor) + \'vh; font-size:\' + (1.5 * fontFactor) + \'vh; font-weight: bold;\'"> {{ area.config.word_post || \'POSTO\' }} <i style="font-size: 1em;" class="fas fa-concierge-bell"></i></th>' +
        ' </tr>' +
        '<template v-for="(ticket, idx) in lastTicketsToShow">' +
          ' <tr :key="idx" v-if="ticket.inactive != \'1\'">' +
          '   <td width="65%" :id="\'queue-color\' + \'-\' + ticket.origin_queue || ticket.queue" :style="\'font-weight: bolder; text-align: center; padding: 2vh 0 2vh 0; background-color: \' + (ticket_called && area.config.show_actual_call * 1 == 1 && (ticket_called.queue == ticket.origin_queue || ticket_called.queue == ticket.queue) && (ticket_called.number == ticket.origin_number || ticket_called.number == ticket.number || ticket_called.number == ticket.last_number_called) ? \'yellow\' : \'white\') + \'; border-radius: 3vh 0 0 3vh; box-shadow: 2px 2px 2px 2px rgba(174,174,174,0.6); font-size:\' + (5 * fontFactor) + \'vh; color: black;\'">{{ hiddeLetter ? \'\' : ticket.origin_queue || ticket.queue }}{{ setFormatQueues(ticket.origin_number || ticket.number || ticket.last_number_called, 8) }}</td>' +
          '   <td width="35%" :style="\'font-weight: bolder; text-align: center; padding: 2vh 0 2vh 0; background-color: \' + getColorQueue(ticket.origin_queue || ticket.queue).background_color + \'; border-radius: 0 3vh 3vh 0; box-shadow: 2px 2px 2px 2px rgba(174,174,174,0.6); font-size:\' + (5 * fontFactor) + \'vh; color: black;\'">{{ ticket.station || ticket.last_station_who_called }}</td>' +
          ' </tr>' +
          '<tr></tr><tr></tr><tr></tr><tr></tr>' +
        '</template>' +
      '</table></center>' +
    '</div>' +
    '<div v-else-if="area.config.type == \'call\' && queues.length > 0 && calledTickets.length > 0">' +
      '<table border=0 v-if="area.config.text_call == \'\'" width="100%">' +
        '<template v-for="(tckt, idx) in calledTickets"">' +
          '<tr>'+
            '<td nowrap :style="\'text-align: center; font-weight: bolder; padding: 2vh; background-color: \' + (idx == 0 ? getColorQueue(tckt.queue).background_color : \'lightgray\') + \'; border-radius: \' + borderRadius + \'vh; box-shadow: \' + (borderRadius ? \'-2px 2px 2px 2px rgba(174,174,174,0.6)\' : \'\') + \'; font-size: \' + (8 * fontFactor) + \'vh; color: \' + (idx == 0 ? getColorQueue(tckt.queue).color : \'black\') + \';\'"><i style="font-size: 1em;" class="fas fa-receipt"></i> {{ hiddeLetter ? \'\' : tckt.queue }}{{ setFormatQueues(tckt.number || tckt.last_number_called, 9) }} &nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <i style="font-size: 1em;" class="fas fa-concierge-bell"></i> {{ tckt.station }}</td>' +
          '</tr>' +
        '</template>' +
      '</table>' +
      '<table border=0 v-else width="100%">' +
        '<template v-for="(tckt, idx) in calledTickets"">' +
          '<tr>'+
            '<td :id="\'identify-call-\' + area.areaKey + \'-\' + lastQueueCalled" nowrap :style="\'text-align: center; font-weight: bolder; padding: 2vh; background-color: \' + (idx == 0 ? getColorQueue(tckt.queue).background_color : \'lightgray\') + \'; border-radius: \' + borderRadius + \'vh; box-shadow: \' + (borderRadius ? \'-2px 2px 2px 2px rgba(174,174,174,0.6)\' : \'\') + \'; font-size: \' + (8 * fontFactor) + \'vh; color: \' + (idx == 0 ? getColorQueue(tckt.queue).color : \'black\') + \';\'"><div v-html="formatTicketToText(tckt)"></div></td>' +
          '</tr>' +
        '</template>' +
      '</table>' +
    '</div>' +
    '<div v-else-if="area.config.type == \'screen\' && area.config.queue_layout == 6 && area.config.show_last_called != 1" class="text-center">' +
      '<q-spinner-ios v-if="ticketPrinting" color="primary" :style="\'font-size: \' + (9 * fontFactor) + \'vh;\'"/>' +
      '<div v-else v-for="queue in queues_with_called" :key="queue">' +
        '<div v-if="queue.inactive != \'1\'"> ' +
          '<div @click="queueClick(3, queue)" v-html="customQueues(queue)"></div>' +
        '</div>' +
      '</div>' +
    '</div>' +
    '<div v-else-if="area.config.type == \'screen\' && area.config.queue_layout == 6 && area.config.show_last_called == 1">' +
      '<div v-for="(ticket, idx) in lastTicketsToShow">' +
        '<div v-if="ticket.inactive != \'1\'"> ' +
          '<div v-html="customQueues(ticket, \'showlastcall\')"></div>' +
        '</div>' +
      '</div>' +
    '</div>' +
  '</div>'
})
// -----------------------------------------------------------------------
// END QUEUES
// -----------------------------------------------------------------------

