// -----------------------------------------------------------------------
// COMPONENT - SLIDESHOW
// -----------------------------------------------------------------------
Vue.component('slideshow', {
  props: {
    area: {
      type: Object
    },
    parent: {
      type: Object
    },
    device_id: {
      type: String
    },
    factor_width: {
      type: Number
    },
    factor_height: {
      type: Number
    }
  },
  data: function () {
    return {
      canSyncContents: true,
      downloadingCount: 0,
      files: [],
      mediasAvailable: [],
      slide: 0,
      timeout: 10000,
      // audioContext: new (window.AudioContext || window.webkitAudioContext)(),
      contentId: '',
      muted: true,
      lastVideoPlayed: '',
      androidFolder: 'Pictures',
      webOSFolder: 'Pictures',
      tizenFolder: 'downloads',
      filesExistInLocalstorage: {},
      filesExistInAndroid: {},
      filesExistInTizen: {},
      filesExistInWebOS: {},
      dont_forward_emission: 0,
      keepInSyncTag: '',
      syncWithWidgets: 0,
      log_content_being_displayed: 0,
      object_fit: 'fill',
      keep_prop: 0,
      shuffle_index: 0,
      shuffle_number: 0,
      shuffle: false,
      useDebug: false,
      timeoutImages: undefined,
      timeoutStart: undefined,
      timeoutSyncStats: undefined,
      videosDuration: {},
      audiosDuration: {},
      videosWhenStarted: {},
      audiosWhenStarted: {},
      customTimeouts: {},
      error: '',
      dbStats: undefined,
      dbContents: undefined,
      timerStats: undefined,
      // timerShowError: undefined,
      borderRadius: '',
      timerDownloadCheck: undefined,
      timerMinuteCheck: undefined,
      downloadingFile: '',
      timerCheckAndDownloadFiles: undefined,
      areaWidth: 0,
      areaHeight: 0,
      elemVideo: '',
      elemImg: '',
      elemAudio: '',
      lastSlideshowRestart: undefined,
      firstRun: true,
      contentBeingPlayed: '',
      downloadingDebug: '',
      timerDownloadingDebug: undefined,
      contentsToIgnore: {}, // used to save video errors and ignore those contents
      loops: 0 // number of loops slideshow already when through
    }
  },
  created: function () {
    var self = this

    self.dbContents = localforage.createInstance({
      name: 'DB_Miravision_Contents',
      version: 1.0,
      driver: [
        localforage.INDEXEDDB,
        localforage.LOCALSTORAGE,
        localforage.WEBSQL,
      ]
    })

    self.dbStats = localforage.createInstance({
      name: 'DB_Miravision_Stats',
      version: 1.0,
      driver: [
        localforage.INDEXEDDB,
        localforage.LOCALSTORAGE,
        localforage.WEBSQL,
      ]
    })

    //
    // 2023-11-10 - THIS IS NOW MADE IN index.js TO AVOID NEED SLIDESHOW
    //

    // in android, we write this file so we can have a confirmation to allow
    // access to write files from android
    /* if (self.parent.isAndroid()) {
      self.cordovaFileExists(self.androidFolder, 'miravision')
      .then(function () {
        // file exists, means we already gave permission to write to disk
        self.parent.debug('File exists...')
      })
      .catch(function () {
        self.parent.debug('File dont exists...')
        var myblob = new Blob(['Miravision'], {
          type: 'text/plain'
        })
        self.cordovaWriteFile(self.androidFolder, 'miravision', myblob)
          .then(function () {
            self.parent.debug('File saved...')
          })
          .catch(function () {
            self.parent.debug('File error saving...')
          })
      })
    } */
  },
  mounted: function () {
    var self = this
    self.setup()
  },
  computed: {
    mediasAvailableImagesFirst: function () {
      var self = this
      if (!Object.keys(self.areaFilesNoFilters).length) return []

      var media = []
      var url = self.parent.http_method + self.parent.apiUrl + '/device_images/' + self.device_id + '/' + self.area.configKey + '/' + self.area.areaKey + '/slideshow/'

      _.forEach(self.areaFilesNoFilters, function (val, key) {
        var arr = key.split(';')
        if (arr.length < 5) return

        var file = arr[4]
        var wmv = file.toLowerCase().indexOf('.wmv')
        var mp4 = file.toLowerCase().indexOf('.mp4')
        var m4v = file.toLowerCase().indexOf('.m4v')
        var mpg = file.toLowerCase().indexOf('.mpg')
        var mpeg = file.toLowerCase().indexOf('.mpeg')
        var jpg = file.toLowerCase().indexOf('.jpg')
        var jpeg = file.toLowerCase().indexOf('.jpeg')
        var png = file.toLowerCase().indexOf('.png')
        var bmp = file.toLowerCase().indexOf('.bmp')
        var avi = file.toLowerCase().indexOf('.avi')
        var gif = file.toLowerCase().indexOf('.gif')
        var wav = file.toLowerCase().indexOf('.wav')
        var mp3 = file.toLowerCase().indexOf('.mp3')

        var elem = {}

        if (wmv != -1 || m4v != -1 || mp4 != -1 || mpg != -1 || mpeg != -1 || avi != -1) {
          elem = {
            id: key,
            type: 'video',
            filename: file,
            url: url + file,
            local_url: ''
          }
        } else if (jpg != -1 || jpeg != -1 || png != -1 || bmp != -1 || gif != -1) {
          elem = {
            id: key,
            type: 'image',
            filename: file,
            url: url + file,
            local_url: ''
          }
        } else if (wav != -1 || mp3 != -1) {
          elem = {
            id: key,
            type: 'audio',
            filename: file,
            url: url + file,
            local_url: ''
          }
        }
        if (Object.keys(elem).length) {
          media.push(elem)
        }
      })

      if (!media.length) return []

      var images = []
      var others = []

      _.forEach(_.orderBy(media, ['type'], ['asc']), function (el, idx) {
        if (el.type == 'image') {
          images.push(el)
        } else {
          others.push(el)
        }
      })

      images = _.orderBy(images, ['filename'], ['asc'])
      others = _.orderBy(others, ['filename'], ['asc'])

      var toReturn = images.concat(others)
      // console.error('mediasAvailableImagesFirst', toReturn)
      return toReturn;
    },
    filesKey: function () {
      var self = this
      return self.device_id + '#' + self.area.configKey + '#' + self.area.areaKey
    },
    areaFilesNoFilters: function () {
      var self = this
      // lets convert parent areaFilesNoFilters from array to obj to be easier
      var myObj = {}
      var areaFilesNoFilters = self.parent.areaFilesNoFilters[self.filesKey]
      if (!areaFilesNoFilters) areaFilesNoFilters = []
      _.forEach(areaFilesNoFilters, function (file) {
        var wmv = file.toLowerCase().indexOf('.wmv')
        var mp4 = file.toLowerCase().indexOf('.mp4')
        var m4v = file.toLowerCase().indexOf('.m4v')
        var mpg = file.toLowerCase().indexOf('.mpg')
        var mpeg = file.toLowerCase().indexOf('.mpeg')
        var jpg = file.toLowerCase().indexOf('.jpg')
        var jpeg = file.toLowerCase().indexOf('.jpeg')
        var png = file.toLowerCase().indexOf('.png')
        var bmp = file.toLowerCase().indexOf('.bmp')
        var avi = file.toLowerCase().indexOf('.avi')
        var gif = file.toLowerCase().indexOf('.gif')
        var wav = file.toLowerCase().indexOf('.wav')
        var mp3 = file.toLowerCase().indexOf('.mp3')

        var id = ''
        var elem = {}

        if (wmv != -1 || m4v != -1 || mp4 != -1 || mpg != -1 || mpeg != -1 || avi != -1) {
          id = self.device_id + ';' + self.area.configKey + ';' + self.area.areaKey + ';video;' + file
        } else if (jpg != -1 || jpeg != -1 || png != -1 || bmp != -1 || gif != -1) {
          id = self.device_id + ';' + self.area.configKey + ';' + self.area.areaKey + ';image;' + file
        } else if (wav != -1 || mp3 != -1) {
          id = self.device_id + ';' + self.area.configKey + ';' + self.area.areaKey + ';audio;' + file
        }
        if (id) {
          myObj[id] = 1
        }
      })
      return myObj
    },
    areaFiles: function () {
      var self = this
      return self.parent.areaFiles[self.filesKey]
    },
    areaFilesTests: function () {
      var self = this
      if (Object.keys(self.parent.areaFiles).length && self.parent.areaFiles.hasOwnProperty(self.filesKey)) {
        var myArr = []
        _.forEach(self.parent.areaFiles[self.filesKey], function (val) {
          myArr.push(val)
        })
        return myArr
      } else {
        return []
      }
    },
    dataFolder: function () {
      var self = this
      var path
      if (self.parent.isElectron()) {
        path = _cdvElectronIpc.dataDir
      } else if (self.parent.isAndroid()) {
        path = cordova.file.externalRootDirectory
      }
      return path
    }
  },
  methods: {
    setup: function (noKeepInSyncStart) {
      var self = this
      // console.log('slideshow - setup()')
      // self.parent.debug('slideshow - setup!')

      self.checkAllowSyncSchedule()

      // operations we want to do every minute
      clearInterval(self.timerMinuteCheck)
      self.timerMinuteCheck = setInterval(function () {
        var second = self.parent.curDate.getSeconds()
        // console.log('second', second)
        if (second == 0) {
          self.checkAllowSyncSchedule()
        }
      }, 1000)

      self.error = 'Waiting for contents...'

      /* clearInterval(self.timerDownloadCheck)
      self.timerDownloadCheck = self.parent.createInterval(function () {
        if (self.downloadingFile != '') {
          self.debug('Downloading file - ' + self.downloadingFile, true)
        }
      }, 10000) */

      /* self.elemVideo = self.area.configKey + '_' + self.area.areaKey + '_vid1'
      self.elemImg = self.area.configKey + '_' + self.area.areaKey + '_img1'
      self.elemAudio = self.area.configKey + '_' + self.area.areaKey + '_audio1' */

      self.elemVideo = 'vid1'
      self.elemImg = 'img1'
      self.elemAudio = 'audio1'

      self.mediasAvailable = []
      self.slide = 0
      self.loops = 0

      self.startArea()

      self.areaWidth = self.area.width * self.factor_width
      self.areaHeight = self.area.height * self.factor_height

      // show initial debug
      setTimeout(function () {
        if (self.downloadingDebug != '') {
          self.parent.debug(self.downloadingDebug)
          self.downloadingDebug = ''
        }
      }, 5000)

      // every 20 secs, show download debug
      clearInterval(self.timerDownloadingDebug)
      self.timerDownloadingDebug = setInterval(function () {
        if (self.downloadingDebug != '') {
          self.parent.debug(self.downloadingDebug)
          self.downloadingDebug = ''
        }
      }, 20000)

      // lets control if slideshow didnt started and try to recover from it
      /* clearInterval(self.timerShowError)
      self.timerShowError = setInterval(function () {
        if (self.error) {
          self.buildMediasAvailable()
          setTimeout(function () {
            if (self.error && !self.downloadingFile && Object.keys(self.parent.slideshowFilesInCache).length == 0) {
              self.parent.debug('No medias available, will reload...')
              setTimeout(function () {
                location.reload()
              }, 4000)
            }
          }, 5000)
        } else {
          clearInterval(self.timerShowError)
        }
      }, 10000) */

      if (!noKeepInSyncStart) {
        // we we're using sync tags to keep slideshow in sync, we need to restart
        if (self.keepInSyncTag != '' && self.downloadingFile == '') {
          setTimeout(function () {
            if (self.mediasAvailable.length > 0 && self.parent.wsConfigConnected) {
              // send an instruction through websocket so all slideshows in all
              // devices restart
              self.parent.restartSlideshows(self.keepInSyncTag)
            }
          }, 3000)
        }
      }
    },
    // control the sync of contents based on a schedule
    checkAllowSyncSchedule: function () {
      var self = this
      // console.log('checkAllowSyncSchedule')
      if (
        self.parent.deviceConfig.config.hasOwnProperty('dates_sync') ||
        self.parent.deviceConfig.config.hasOwnProperty('times_sync') ||
        self.parent.deviceConfig.config.hasOwnProperty('week_days_sync')
      ) {
        var payload = {
          dates: self.parent.deviceConfig.config.dates_sync || [],
          times: self.parent.deviceConfig.config.times_sync || [],
          week_days: self.parent.deviceConfig.config.week_days_sync || []
        }
        if (!self.parent.checkShow(payload)) {
          self.canSyncContents = false
          // console.log('nao sincroniza!')
        } else {
          // console.log('self.canSyncContents', self.canSyncContents)
          // sync was disabled and now is enabled. lets reload to try to download
          // new contents
          if (! self.canSyncContents) {
            location.reload()
          }
          self.canSyncContents = true
          // console.log('sincroniza!')
        }
      }
      // self.canSyncContents = true
      // console.log('self.canSyncContents', self.canSyncContents)
    },
    doClick: function () {
      var self = this
      var screen_to = self.area.config.screen_to || ''
      if (screen_to == '') return
      var img1 = self.$refs[self.elemImg]
      var vid1 = self.$refs[self.elemVideo]
      var audio1 = self.$refs[self.elemAudio]
      if (img1) img1.style.display = 'none'
      if (vid1) vid1.style.display = 'none'
      if (audio1) {
        // self.debug('pause2')
        audio1.pause()
      }
      self.$emit('screen-to', screen_to)
    },
    handleObjectFit: function (width, height) {
      var self = this
      // console.log('handleObjectFit', width + 'x' + height, self.keep_prop)
      if (self.keep_prop == 1) {
        self.object_fit = 'contain'
      } else {
        self.object_fit = 'fill'
        /* if (self.areaWidth > self.areaHeight) {
          // horizontal area, width can't be bigger than height, force object_fit
          if (height > width) {
            self.object_fit = 'contain'
          }
        } else {
          // vertical area, height can't be bigger than width, force object_fit
          if (width > height) {
            self.object_fit = 'contain'
          }
        } */
      }
    },
    startArea: function () {
      var self = this
      // console.log('startArea', self.area.configKey, self.area.areaKey)

      var img1 = self.$refs[self.elemImg]
      var vid1 = self.$refs[self.elemVideo]
      var audio1 = self.$refs[self.elemAudio]

      if (img1) img1.style.display = 'none'
      if (vid1) vid1.style.display = 'none'
      if (audio1) audio1.pause()

      var cfg = self.area.config
      if (cfg.hasOwnProperty('slide_timeout') && cfg.slide_timeout) self.timeout = cfg.slide_timeout * 1000
      if (cfg.hasOwnProperty('play_sound') && cfg.play_sound) self.muted = false
      if (cfg.hasOwnProperty('show_debug_info') && cfg.show_debug_info * 1 == 1) self.useDebug = true
      if (cfg.hasOwnProperty('content_id') && cfg.content_id) self.contentId = cfg.content_id
      if (cfg.hasOwnProperty('dont_forward_emission')) self.dont_forward_emission = cfg.dont_forward_emission * 1
      if (cfg.hasOwnProperty('keep_in_sync_tag')) self.keepInSyncTag = cfg.keep_in_sync_tag
      if (cfg.hasOwnProperty('sync_with_widgets')) self.syncWithWidgets = cfg.sync_with_widgets * 1
      if (cfg.hasOwnProperty('log_content_being_displayed')) self.log_content_being_displayed = cfg.log_content_being_displayed * 1
      if (cfg.hasOwnProperty('border_radius') && cfg.border_radius * 1 > 0) self.borderRadius = 'border-radius: ' + cfg.border_radius + 'vh;'
      if (cfg.hasOwnProperty('shuffle') && cfg.shuffle * 1 == 1) self.shuffle = true
      if (cfg.hasOwnProperty('shuffle_number')) self.shuffle_number = cfg.shuffle_number * 1
      if (cfg.hasOwnProperty('keep_prop')) self.keep_prop = cfg.keep_prop * 1

      self.customTimeouts = {}

      if (cfg.hasOwnProperty('slide_timeout_custom')) {
        var custom = cfg.slide_timeout_custom.split(',')
        if (custom.length > 0) {
          for (var i = 0; i < custom.length; i++) {
            if (custom[i]) self.customTimeouts[i] = (custom[i] * 1) * 1000
          }
        }
      }

      // we start the sync stats procedure 10 secs after start so we have time to
      // know if we've internet or not
      clearTimeout(self.timeoutSyncStats)
      self.timeoutSyncStats = setTimeout(function () {
        if (self.contentId) {
          self.syncStats()
          clearInterval(self.timerStats)
          self.timerStats = self.parent.createInterval(function () {
            self.syncStats()
          }, self.parent.getRandomInt(300, 600) * 1000)
        }
      }, 10000)

      if (self.parent.isAndroid()) {
        self.cordovaDeleteInvalidFiles(self.androidFolder)
      } else if (self.parent.isTizen()) {
        self.tizenDeleteInvalidFiles(self.tizenFolder)
      }

      if (!self.parent.checkedDownloadFiles.hasOwnProperty(self.filesKey)) {
        // build data structure but don't start player
        self.buildMediasAvailable(true)
        // if (self.parent.hasInternet) {
        self.parent.checkedDownloadFiles[self.filesKey] = 1
        self.checkAndDownloadFiles()
        // }
        clearTimeout(self.startAreaTimeout)
        self.startAreaTimeout = setTimeout(function () {
          self.buildMediasAvailable()
        }, 1000)

        // self.contentCleanups()
      } else {
        self.buildMediasAvailable()
      }
    },
    syncStats: function () {
      var self = this
      if (!self.parent.hasInternet) {
        self.debug('cant sync stats because there is no Internet connection')
        return
      }
      self.dbStats.getItem('stats').then(function (stats) {
        self.debug(stats.length + ' stats to sync...')
        var curStats = []
        if (stats) curStats = stats
        if (curStats.length == 0) return
        // clean the actual stats
        var url = self.parent.http_method + self.parent.apiUrl + '/stats_collect_form_method'
        var xhr = new XMLHttpRequest()
        xhr.onreadystatechange = function () {
          if (xhr.readyState == XMLHttpRequest.DONE) {
            if (xhr.status == 200) {
              self.dbStats.setItem('stats', []).then(function (stats) { })
              // self.debug('STATS SYNC SUCCESS - ' + xhr.responseText)
            } else {
              // self.debug('STATS SYNC ERROR - ' + xhr.statusText)
            }
          }
        }
        var toSend = {
          device_id: self.device_id,
          stats: curStats
        }
        xhr.open('POST', url)
        var data = new FormData()
        data.append('json', JSON.stringify(toSend))
        xhr.send(data)
      })
    },
    debug: function (msg, force) {
      var self = this
      console.log(msg)
      if (self.useDebug || force) {
        self.$emit('debug', 'slideshow - ' + msg)
      }
    },
    buildMediasAvailable: function (dontStartPlaying, checkContentBeingPlayed) {
      var self = this

      // console.log('buildMediasAvailable, dontStartPlaying: ' + dontStartPlaying + ', checkContentBeingPlayed: ' + checkContentBeingPlayed)
      // console.log('buildMediasAvailable, areaFiles', self.areaFiles)

      // we had no contents, allways restart
      if (self.mediasAvailable.length == 0) {
        dontStartPlaying = false
      }

      // this timeout is needed because sometimes self.areaFiles wouldn't reflect
      // whats happening in self.parent.areaFiles, this gives time to vue to update
      // parent values
      setTimeout(function () {
        var media = []
        var url = self.parent.http_method + self.parent.apiUrl + '/device_images/' + self.device_id + '/' + self.area.configKey + '/' + self.area.areaKey + '/slideshow/'
        _.forEach(self.areaFiles, function (file, idx) {
          var wmv = file.toLowerCase().indexOf('.wmv')
          var mp4 = file.toLowerCase().indexOf('.mp4')
          var m4v = file.toLowerCase().indexOf('.m4v')
          var mpg = file.toLowerCase().indexOf('.mpg')
          var mpeg = file.toLowerCase().indexOf('.mpeg')
          var jpg = file.toLowerCase().indexOf('.jpg')
          var jpeg = file.toLowerCase().indexOf('.jpeg')
          var png = file.toLowerCase().indexOf('.png')
          var bmp = file.toLowerCase().indexOf('.bmp')
          var avi = file.toLowerCase().indexOf('.avi')
          var gif = file.toLowerCase().indexOf('.gif')
          var wav = file.toLowerCase().indexOf('.wav')
          var mp3 = file.toLowerCase().indexOf('.mp3')

          var id = ''
          var elem = {}

          if (wmv != -1 || m4v != -1 || mp4 != -1 || mpg != -1 || mpeg != -1 || avi != -1) {
            id = self.device_id + ';' + self.area.configKey + ';' + self.area.areaKey + ';video;' + file
            elem = {
              id: id,
              type: 'video',
              filename: file,
              url: url + file,
              local_url: ''
            }
          } else if (jpg != -1 || jpeg != -1 || png != -1 || bmp != -1 || gif != -1) {
            id = self.device_id + ';' + self.area.configKey + ';' + self.area.areaKey + ';image;' + file
            elem = {
              id: id,
              type: 'image',
              filename: file,
              url: url + file,
              local_url: ''
            }
          } else if (wav != -1 || mp3 != -1) {
            id = self.device_id + ';' + self.area.configKey + ';' + self.area.areaKey + ';audio;' + file
            elem = {
              id: id,
              type: 'audio',
              filename: file,
              url: url + file,
              local_url: ''
            }
          }
          if (id && !self.contentsToIgnore.hasOwnProperty(id)) {
            elem['local_url'] = ''
            if (self.parent.isLocalhost) {
              var localUrl = elem.url
              localUrl = _.replace(localUrl, 'apis.opalaconsult.com', '127.0.0.1')
              elem['local_url'] = localUrl
            } else if (self.parent.isAndroid()) {
              if (self.filesExistInAndroid.hasOwnProperty(id)) {
                var path = self.dataFolder + self.androidFolder + '/' + id
                elem['local_url'] = path
              }
            } else if (self.parent.isTizen()) {
              if (self.filesExistInTizen.hasOwnProperty(id)) {
                var path = ''
                // in tizen 2.4, the folder is different
                if (_.includes(self.parent.isTizen() && self.parent.tvFirmwareVersion, '2.4')) {
                  path = 'file:///opt/usr/media/Downloads/' + id
                } else {
                  path = 'file:///opt/usr/home/owner/content/Downloads/' + id
                }
                elem['local_url'] = path
              }
              //} else if (self.parent.isWebOS()) {
              //  self.debug('need to implement load local file in WEBOS')
            } else {
              if (self.parent.slideshowFilesInCache.hasOwnProperty(id)) {
                var path = self.parent.slideshowFilesInCache[id]
                elem['local_url'] = path
              }
            }
            if (elem['local_url'] != '') media.push(elem)
          }
        })

        // self.parent.debug('buildMediasAvailable - media.length - ' + media.length)
        // console.log('buildMediasAvailable - media.length - ' + media.length)

        var needStartPlaying = false

        if (media.length > 0) {
          self.error = ''
          // self.debug('MEDIA - ' + media.length)
          // self.parent.debug('MEDIA - ' + JSON.stringify(media))
          var contentsArray = _.orderBy(media, ['filename'], ['asc'])
          var use_n_musics = self.area.config.use_n_musics || 0
          if (use_n_musics > 0) contentsArray = _.take(contentsArray, use_n_musics)
          if (self.shuffle && contentsArray.length > 1) contentsArray = _.shuffle(contentsArray)
          self.mediasAvailable = contentsArray
          if (!dontStartPlaying) {
            needStartPlaying = true
          }
        } else {
          self.mediasAvailable = []
          self.debug('No medias available yet...')
        }

        if (needStartPlaying) {
          self.startPlaying()
        }
        else if (checkContentBeingPlayed) {
          // lets check if the content being played is still in the playlist, if not
          // we'll start from beggining
          if (self.contentBeingPlayed != '') {
            var found = false
            _.forEach(self.mediasAvailable, function (el) {
              if (el.local_url == self.contentBeingPlayed) {
                found = true
                return false
              }
            })
            // the means the content being played has disapeared (sheduling maybe)
            // lets start again
            if (!found) {
              self.startPlaying()
            }
          }
        }
        // console.error('contentBeingPlayed', self.contentBeingPlayed)
        // console.error('buildMediasAvailable() - self.mediasAvailable', self.mediasAvailable)
      }, 100)
    },
    contentCleanups: function () {
      var self = this
      // console.log('contentCleanups', self.parent.playlistsCacheControl)
      if (self.parent.isAndroid()) {
        // self.debug('contentCleanups - android')
        self.cordovaFolderFilesAll(self.androidFolder).then(function (files) {
          _.forEach(files, function (file) {
            var arr = file.split(';')
            if (arr.length >= 5) {
              var keyToCheck = arr[0] + '#' + arr[1] + '#' + arr[2]
              if (!self.parent.playlistsCacheControl.hasOwnProperty(keyToCheck)) {
                self.debug('remove - ' + file)
                self.cordovaFileDelete(self.androidFolder, file).then(function () { })
              }
            }
          })
        }, function () {
        })
      } else if (self.parent.isTizen()) {
        // self.debug('contentCleanups - tizen')
        self.tizenFolderFilesAll(self.tizenFolder).then(function (files) {
          setTimeout(function () {
            _.forEach(files, function (file) {
              var arr = file.split(';')
              if (arr.length >= 5) {
                var keyToCheck = arr[0] + '#' + arr[1] + '#' + arr[2]
                if (!self.parent.playlistsCacheControl.hasOwnProperty(keyToCheck)) {
                  self.debug('remove - ' + file)
                  self.tizenFileDelete(self.tizenFolder, file).then(function () { })
                }
              }
            })
          }, 100)
        }, function () {
        })
        // } else if (self.parent.isWebOS()) {
        // } else if (self.parent.isTizen() || self.parent.isWebOS() || self.parent.isLinux()) {
      } else if (!self.parent.isLocalhost) {
        // self.debug('contentCleanups - linux or webos')
        // keep cache clean and remove non used elements anymore
        var keysToCleanup = []
        self.dbContents.iterate(function (value, key, iterationNumber) {
          var arr = key.split(';')
          if (arr.length >= 5) {
            var keyToCheck = arr[0] + '#' + arr[1] + '#' + arr[2]
            if (!self.parent.playlistsCacheControl.hasOwnProperty(keyToCheck)) {
              keysToCleanup.push(key)
            }
          }
        }).then(function () {
          _.forEach(keysToCleanup, function (v) {
            self.debug('remove - ' + v)
            self.dbContents.removeItem(v)
          })
        })
      }
    },
    checkAndDownloadFiles: function () {
      var self = this
      // self.parent.debug('checkAndDownloadFiles!!!!!')
      if (self.parent.isAndroid()) {
        // self.debug('files download is not ready for android yet... - ' + db)
        self.cordovaFolderFiles(self.androidFolder).then(function (files) {
          self.cordovaCheckAndDownloadFiles(files)
        }, function () {
          self.cordovaCheckAndDownloadFiles([])
        })
      } else if (self.parent.isTizen()) {
        self.tizenFolderFiles(self.tizenFolder).then(function (files) {
          self.tizenCheckAndDownloadFiles(files)
        }, function () {
          self.tizenCheckAndDownloadFiles([])
        })
        // } else if (self.parent.isWebOS()) {
        //   self.webOSFolderFiles(self.webOSFolder).then(function (files) {
        //     self.webOSCheckAndDownloadFiles(files)
        //   }, function () {
        //     self.webOSCheckAndDownloadFiles([])
        //   })
        // } else if (self.parent.isTizen() || self.parent.isWebOS() || self.parent.isLinux()) {
      } else if (!self.parent.isLocalhost) {
        var keysInDb = {}
        var keysInMedia = {}
        // Use the iterate method to get all items in the database
        self.dbContents.iterate(function (value, key, iterationNumber) {
          keysInDb[key] = 1
        }).then(function () {
          // delete files that aren't in the actual playlist (including the contents
          // that are scheduled but not being used)
          filesToDelete = {}
          _.forEach(keysInDb, function (el, key) {
            var arr = _.split(key, ';')
            if (arr.length < 4) {
              filesToDelete[key] = 1
            } else if (arr[0] != self.device_id) {
              filesToDelete[key] = 1
            } else if (!self.parent.playListValidAreasByKey.hasOwnProperty(arr[1] + '#' + arr[2])
            ) {
              filesToDelete[key] = 1
            } else if (arr[0] == self.device_id && arr[1] == self.area.configKey && arr[2] == self.area.areaKey) {
              // means there's a content in database that's not in the media, remove it
              if (!self.areaFilesNoFilters.hasOwnProperty(key)) {
                filesToDelete[key] = 1
              }
            }
          })

          if (self.canSyncContents) {
            _.forEach(filesToDelete, function (el, key) {
              self.dbContents.removeItem(key).then(function () {
                // self.parent.debug('file delete - ' + key)
              })
            })
          }

          // _.forEach(_.orderBy(self.mediasAvailable, ['type'], ['asc']), function (el, idx) {
          _.forEach(self.mediasAvailableImagesFirst, function (el, idx) {
            if (self.parent.slideshowFilesInCache.hasOwnProperty(el.id)) return true
            self.dbContents.getItem(el.id).then(function (image) {
              if (!image) {
                // self.debug('DONT EXISTS! ' + el.id)
                if (self.canSyncContents) {
                  self.downloadFile(el)
                }
              } else {
                // self.debug('EXISTS! ' + el.id)
                var blob = new Blob([image])
                var imageURI = window.URL.createObjectURL(blob)
                self.$set(self.parent.slideshowFilesInCache, el.id, imageURI)
              }
            }).catch(function (err) {
              // indexdb error, ignore
              self.debug('ERROR! ' + err.message)
            })
          })
          // console.log('keysInDb', keysInDb)
          // console.log('keysInMedia', keysInMedia)
        }).catch(function (err) {
          // console.error('Error while iterating: ' + err)
        })
      }
    },
    tizenCheckAndDownloadFiles: function (files) {
      var self = this
      var filesToDelete = {}
      var areaFiles = {}

      // self.parent.debug('files - ' + self.area.areaKey + ' - ' + JSON.stringify(files))

      _.forEach(files, function (el) {
        var arr = el.split(';')
        if (arr.length < 4) {
          filesToDelete[el] = 1
        } else if (arr[0] != self.device_id) {
          filesToDelete[el] = 1
        } else if (!self.parent.playListValidAreasByKey.hasOwnProperty(arr[1] + '#' + arr[2])
        ) {
          filesToDelete[el] = 1
        }
      })

      _.forEach(self.mediasAvailableImagesFirst, function (el) {
        areaFiles[el.id] = 1
      })

      /* if (self.area.areaKey == 'area2') {
        self.parent.debug(self.area.areaKey + ' - areaFiles - ' + JSON.stringify(areaFiles))
      } */

      _.forEach(files, function (el) {
        if (filesToDelete.hasOwnProperty(el)) return
        var arr = el.split(';')
        if (
          arr.length >= 4 &&
          arr[0] == self.device_id &&
          arr[1] == self.area.configKey &&
          arr[2] == self.area.areaKey &&
          !areaFiles.hasOwnProperty(el)
        ) {
          filesToDelete[el] = 1
        } else {
          self.$set(self.filesExistInTizen, el, 1)
        }
      })

      if (self.canSyncContents) {
        _.forEach(filesToDelete, function (el, key) {
          // self.parent.debug('file delete - ' + key)
          self.tizenFileDelete(self.tizenFolder, key).then(function () {
          })
          self.tizenFileDelete(self.tizenFolder, key + '.__mira_completed').then(function () {
          })
        })
      }

      _.forEach(self.mediasAvailableImagesFirst, function (el, idx) {
        if (!self.filesExistInTizen.hasOwnProperty(el.id)) {
          if (self.canSyncContents) {
            // file don't exist yet, we need to download it
            self.tizenFileDownload(el)
          }
        } else {
          self.$set(self.parent.slideshowFilesInCache, el.id, 1)
        }
      })
    },
    tizenFileDelete: function (path, filename) {
      var self = this
      return new Promise(function (resolve, reject) {
        // self.debug('tizenFileDelete - ' + path + ' - ' + filename)
        tizen.filesystem.resolve(
          path,
          function (dir) {
            dir.deleteFile(
              path + '/' + filename,
              function () {
                resolve(filename)
              }, function (e) {
                self.debug('aqui2.2 - ' + filename + ' - ' + e.message)
                reject(e)
              }
            )
          }, function (e) {
            reject(e)
          },
          'rw'
        )
      })
    },
    tizenFileDownload: function (el) {
      var self = this
      if (self.downloadingFile != '') return
      if (self.parent.hasInternet == false) return
      if (self.filesExistInTizen[el.id]) return // make sure we dont download twice

      // we need to remove the file extension because tizen 2.4 auto adds another
      // file extension to the downloaded file... don't know why...
      // it works in modern tizens also
      var fname = el.id
      var myArr = fname.split('.')
      myArr.pop()
      fname = myArr.join('.')

      var req = new tizen.DownloadRequest(el.url, self.tizenFolder, fname, 'ALL')
      // req.httpHeader['Pragma'] = 'no-cache'
      var listener = {
        onpaused: function (id) {
          self.debug('TIZEN DOWNLOAD PAUSED - ' + el.id, 'red')
          location.reload()
        },
        oncanceled: function (id) {
          self.debug('TIZEN DOWNLOAD CANCELED - ' + el.id, 'red')
          delete self.parent.slideshowFilesInCache[el.id]
          self.downloadingFile = ''
          self.tizenFolderFiles(self.tizenFolder).then(function (files) {
            self.tizenCheckAndDownloadFiles(files)
          }, function () {
            self.tizenCheckAndDownloadFiles([])
          })
        },
        oncompleted: function (id, fullPath) {
          self.debug('download complete - ' + el.filename)
          self.filesExistInTizen[el.id] = 1
          // self.debug('TIZEN DOWNLOAD - oncompleted: ' + id + ' - ' + fullPath)
          var myblob = new Blob(['Download done...'], { type: 'text/plain' })
          self.tizenWriteFile(self.tizenFolder, el.id + '.__mira_completed', myblob).then(function () { })
          self.downloadingFile = ''
          setTimeout(function () {
            self.buildMediasAvailable(true)
            self.tizenFolderFiles(self.tizenFolder).then(function (files) {
              self.tizenCheckAndDownloadFiles(files)
            }, function () {
              self.tizenCheckAndDownloadFiles([])
            })
          }, 1000)
        },
        onfailed: function (error) {
          self.debug('TIZEN DOWNLOAD FAILED - ' + el.id, 'red')
          delete self.parent.slideshowFilesInCache[el.id]
          self.downloadingFile = ''
          self.tizenFolderFiles(self.tizenFolder).then(function (files) {
            self.tizenCheckAndDownloadFiles(files)
          }, function () {
            self.tizenCheckAndDownloadFiles([])
          })
        }
      }
      var id = tizen.download.start(req, listener)
      self.downloadingFile = el.filename
      self.debug('Will download - ' + self.area.configKey + '#' + self.area.areaKey + ' - ' + el.filename)
    },
    tizenLoadFile: function (filePath) {
      var self = this
      self.debug('tizenLoadFile - ' + filePath)
      var req = new XMLHttpRequest()
      req.responseType = 'arraybuffer'
      req.addEventListener('load', function (evt) {
        var buffer = evt.target.response
        // self.debug('CONSEGUI LER!!!!!!')
        // processRawData(buffer)
      })
      req.addEventListener('error', function (e) { })
      req.open('GET', filePath)
      req.send()
    },
    stringToArrayBuffer: function (string) {
      var buffer = new ArrayBuffer(string.length)
      var view = new Uint8Array(buffer)
      for (var i = 0; i < string.length; i++) {
        view[i] = string.charCodeAt(i)
      }
      return view
    },
    tizenDeleteInvalidFiles: function (path) {
      var self = this
      tizen.filesystem.resolve(
        self.tizenFolder,
        function (dir) {
          dir.listFiles(function (files) {
            if (files.length == 0) {
              self.debug('TIZEN - NO FILES IN DEVICE...')
            } else {
              var downloadsCompleted = {}
              // self.debug('files - ' + files.length)
              for (var i = 0; i < files.length; i++) {
                var file = files[i]
                var fname = file.name
                if (_.includes(fname, '.__mira_completed')) {
                  fname = _.replace(fname, '.__mira_completed', '')
                  downloadsCompleted[fname] = 1
                }
                // self.debug('TIZEN FILE IN HDD - ' + file.fileSize + ' - ' + file.name)
                // }
                // self.tizenFileDelete('downloads', file.name).then(function (fileDeleted) {})
              }
              for (var i = 0; i < files.length; i++) {
                var file = files[i]
                var fname = file.name
                var arr = _.split(fname, ';')
                // its a miravision file
                if (arr.length >= 4) {
                  if (!_.includes(fname, '.__mira_completed') && !downloadsCompleted.hasOwnProperty(fname)) {
                    // self.debug('content deleted - ' + fname, true)
                    self.tizenFileDelete(self.tizenFolder, fname).then(function () {
                    })
                    self.tizenFileDelete(self.tizenFolder, fname + '.__mira_completed').then(function () {
                    })
                  }
                }
              }
            }
          })
        },
        function (e) {
          self.debug('Error - ' + e.message);
        },
        'rw'
      )
    },
    tizenDeleteAllFiles: function () {
      tizen.filesystem.resolve(
        self.tizenFolder,
        function (dir) {
          dir.listFiles(function (files) {
            // self.debug('files - ' + files.length)
            for (var i = 0; i < files.length; i++) {
              var file = files[i]
              // self.debug('file - ' + file.fileSize + ' - ' + file.name)
              self.tizenFileDelete(self.tizenFolder, file.name).then(function (fileDeleted) {
                // self.debug('file deleted - ' + fileDeleted)
              })
            }
          })
        },
        function (e) {
          self.debug('Error - ' + e.message);
        },
        'rw'
      )
    },
    cordovaCheckAndDownloadFiles: function (files) {
      var self = this
      var filesToDelete = {}
      var areaFiles = {}

      // self.parent.debug('files - ' + self.area.areaKey + ' - ' + JSON.stringify(files))

      _.forEach(files, function (el) {
        var arr = el.split(';')
        if (arr.length < 4) {
          filesToDelete[el] = 1
        } else if (arr[0] != self.device_id) {
          filesToDelete[el] = 1
        } else if (!self.parent.playListValidAreasByKey.hasOwnProperty(arr[1] + '#' + arr[2])
        ) {
          filesToDelete[el] = 1
        }
      })

      _.forEach(self.mediasAvailableImagesFirst, function (el) {
        areaFiles[el.id] = 1
      })

      _.forEach(files, function (el) {
        if (filesToDelete.hasOwnProperty(el)) return
        var arr = el.split(';')
        if (
          arr.length >= 4 &&
          arr[0] == self.device_id &&
          arr[1] == self.area.configKey &&
          arr[2] == self.area.areaKey &&
          !areaFiles.hasOwnProperty(el)
        ) {
          filesToDelete[el] = 1
        } else {
          self.$set(self.filesExistInAndroid, el, 1)
        }
      })

      if (self.canSyncContents) {
        _.forEach(filesToDelete, function (el, key) {
          // self.parent.debug('file delete - ' + key)
          self.cordovaFileDelete(self.androidFolder, key).then(function () {
          })
          self.cordovaFileDelete(self.androidFolder, key + '.__mira_completed').then(function () {
          })
        })
      }

      _.forEach(self.mediasAvailableImagesFirst, function (el, idx) {
        if (!self.filesExistInAndroid.hasOwnProperty(el.id)) {
          if (self.canSyncContents) {
            // file don't exist yet, we need to download it
            self.cordovaFileDownload(el)
          }
        } else {
          self.$set(self.parent.slideshowFilesInCache, el.id, 1)
        }
      })
    },
    /* cordovaCheckAndDownloadFilesOld: function (files) {
      var self = this
      var deviceFiles = {}

      _.forEach(files, function (el, idx) {
        self.$set(self.filesExistInAndroid, el, 1)
      })

      _.forEach(files, function (el, idx) {
        var fnameOrig = el
        var fname = el
        // lets cleanup all mira_completed files that has no real file
        if (_.includes(fname, '.__mira_completed')) {
          fname = _.replace(fname, '.__mira_completed', '')
          self.cordovaFileExists(self.androidFolder, fname)
          .then(function () {
            // file exists
          })
          .catch(function () {
            self.cordovaFileDelete(self.androidFolder, fnameOrig).then(function () {
              // alert(key + ' - deleted1')
            })
          })
        }
        self.$set(self.filesExistInAndroid, el, 1)
      })

      // make sure images come first
      // _.forEach(_.orderBy(self.mediasAvailable, ['type'], ['asc']), function (el, idx) {
      _.forEach(self.mediasAvailableImagesFirst, function (el, idx) {
        deviceFiles[el.id] = 1
        if (!self.filesExistInAndroid.hasOwnProperty(el.id)) {
          // file don't exist yet, we need to download it
          self.cordovaFileDownload(el)
        }
      })

      var keysToRemove = []

      // make sure we deleted the files we're not using anymore
      _.forEach(self.filesExistInAndroid, function (el, key) {
        var arr = el.name.split(';')
        if (arr.length >= 4 && arr[0] == self.device_id && arr[1] == self.area.configKey && arr[2] == self.area.areaKey) {
          if (!deviceFiles.hasOwnProperty(key)) {
            // self.debug('ANDROID FILE DELETE - ' + key)
            keysToRemove.push(key)
            // alert('need to remove file - ' + key)
            self.cordovaFileDelete(self.androidFolder, key).then(function () {
              // alert(key + ' - deleted1')
            })
            self.cordovaFileDelete(self.androidFolder, key + '.__mira_completed').then(function () {
              // alert(key + ' - deleted2')
            })
          }
        }
      })

      _.forEach(keysToRemove, function (el, idx) {
        _.omit(self.filesExistInAndroid, [el])
      })
    }, */
    cordovaFileDownload: function (el) {
      var self = this
      if (self.downloadingFile != '') return
      if (self.parent.hasInternet == false) return
      if (self.filesExistInAndroid[el.id]) return // make sure we dont download twice
      self.downloadingFile = el.filename
      self.debug('Will download - ' + self.area.configKey + '#' + self.area.areaKey + ' - ' + el.filename)

      var fileTransfer = new FileTransfer()
      var uri = encodeURI(el.url)
      var filePath = self.dataFolder + self.androidFolder + '/' + el.id

      fileTransfer.download(
        uri,
        filePath,
        function (entry) {
          self.debug('download complete - ' + el.filename)

          self.filesExistInAndroid[el.id] = 1
          // main file
          // self.cordovaWriteFile(self.androidFolder, el.id, blob)

          // done file
          var myblob = new Blob(['Download done...'], {
            type: 'text/plain'
          })
          self.cordovaWriteFile(self.androidFolder, el.id + '.__mira_completed', myblob)
          self.downloadingFile = ''

          setTimeout(function () {
            self.buildMediasAvailable(true)
            self.cordovaFolderFiles(self.androidFolder).then(function (files) {
              self.cordovaCheckAndDownloadFiles(files)
            }, function () {
              self.cordovaCheckAndDownloadFiles([])
            })
          }, 1000)
        },
        function (error) {
          self.debug('Download error: ' + el.filename + ' - ' + error.code, 'red')
          delete self.parent.slideshowFilesInCache[el.id]
          self.downloadingFile = ''
          self.cordovaCheckAndDownloadFiles()
        },
        false, // set to true to use HTTPS
        {
          headers: {}
        }
      )
    },
    // deletes files that have 0 size
    cordovaDeleteInvalidFiles: function (path) {
      var self = this
      window.resolveLocalFileSystemURL(self.dataFolder + path, function (fs) {
        var reader = fs.createReader()
        reader.readEntries(function (files) {
          var downloadsCompleted = {}
          _.forEach(files, function (el, idx) {
            if (el.isFile) {
              var fname = el.name
              if (_.includes(fname, '.__mira_completed')) {
                fname = _.replace(fname, '.__mira_completed', '')
                downloadsCompleted[fname] = 1
              }
            }
          })
          _.forEach(files, function (el, idx) {
            if (el.isFile) {
              var arr = el.name.split(';')
              // its a miravision file
              if (arr.length >= 4) {
                el.getMetadata(function (metadata) {
                  // alert(el.name + ' - ' + metadata.size)
                  if (metadata.size == 0) {
                    _.omit(downloadsCompleted, [el.name])
                    self.cordovaFileDelete(self.androidFolder, el.name).then(function () {
                    })
                    self.cordovaFileDelete(self.androidFolder, el.name + '.__mira_completed').then(function () {
                    })
                  } else {
                    var fname = el.name
                    var arr = _.split(el.name, ';')
                    // its a miravision file
                    if (arr.length >= 4) {
                      if (!_.includes(fname, '.__mira_completed') && !downloadsCompleted.hasOwnProperty(fname)) {
                        // self.debug('content deleted - ' + fname, true)
                        self.cordovaFileDelete(self.androidFolder, fname).then(function () {
                        })
                        self.cordovaFileDelete(self.androidFolder, fname + '.__mira_completed').then(function () {
                        })
                      }
                    }
                  }
                })
              }
            }
          })
        })
      })
    },
    cordovaFileDelete: function (path, filename) {
      var self = this
      return new Promise(function (resolve, reject) {
        window.resolveLocalFileSystemURL(self.dataFolder, function (dirpar) {
          dirpar.getDirectory(path, { create: true }, function (dir) {
            dir.getFile(filename, { create: false, exclusive: false }, function (fileEntry) {
              fileEntry.remove(function () {
                resolve()
              }, reject)
            }, reject)
          }, reject)
        }, reject)
      })
    },
    cordovaFileExists: function (path, filename) {
      const self = this
      // self.parent.debug('cordovaFileExists: ' + self.dataFolder + ' - ' + path)
      return new Promise(function (resolve, reject) {
        window.resolveLocalFileSystemURL(self.dataFolder, function (dirpar) {
          dirpar.getDirectory(path, { create: true }, function (dir) {
            dir.getFile(filename, { create: false, exclusive: false }, function (fileEntry) {
              fileEntry.getMetadata(function (metadata) {
                /* {
                  modificationTime: '2022-07-26T16:36:31.000Z',
                  size: 903672
                  } */
                if (metadata.size * 1 > 0) {
                  resolve()
                } else {
                  reject()
                }
              }, reject)
            }, reject)
          }, reject)
        }, reject)
      })
    },
    cordovaWriteFile: function (path, filename, blob) {
      var self = this
      return new Promise(function (resolve, reject) {
        window.resolveLocalFileSystemURL(self.dataFolder, function (dirpar) {
          dirpar.getDirectory(path, { create: true }, function (dir) {
            dir.getFile(filename, { create: true, exclusive: false }, function (fileEntry) {
              // here comes
              fileEntry.createWriter(function (fileWriter) {
                // append to the end
                fileWriter.seek(fileWriter.length)
                var finishedWriting = function () {
                  resolve(fileEntry)
                }
                var offset = 0
                // var BLOCK_SIZE = 1 * 1024 * 1024 // write blocks of 1MB at a time
                var BLOCK_SIZE = 1 * 5120 * 1024 // write blocks of 5MB at a time
                var writeNext = function (finishCallback) {
                  var blockSize = Math.min(BLOCK_SIZE, blob.size - offset)
                  var block = blob.slice(offset, offset + blockSize)
                  fileWriter.onwriteend = function () {
                    if (offset > 0) {
                      if (!filename.includes('.__mira_completed')) {
                        if (parseInt(offset / 5120 / 1024) > 0) {
                          var arr = filename.split(';')
                        }
                      }
                    }
                    if (offset < blob.size) {
                      offset += blockSize
                      writeNext(finishCallback)
                    } else {
                      finishCallback()
                    }
                  }
                  fileWriter.write(block)
                }
                writeNext(finishedWriting)
              }, function (err) {
                reject('error writing: ' + err)
              })
              // here finishes
            }, reject)
          }, reject)
        }, reject)
      })
    },
    /* webOSCheckAndDownloadFiles: function () {
      var self = this
      self.debug('webOSCheckAndDownloadFiles')
    },
    webOSDeleteInvalidFiles: function () {
      var self = this
      self.debug('webOSDeleteInvalidFiles')
    },
    webOSFileDelete: function () {
      var self = this
      self.debug('webOSFileDelete')
    },
    webOSFileDownload: function () {
      var self = this
      self.debug('webOSFileDownload')
    },
    webOSFileExists: function () {
      var self = this
      self.debug('webOSFileExists')
    },
    webOSFolderFiles: function () {
      var self = this
      self.debug('webOSFolderFiles')
      return new Promise(function (resolve, reject) {
        resolve()
      })
    },
    webOSFolderFilesAll: function () {
      var self = this
      self.debug('webOSFolderFiles')
      return new Promise(function (resolve, reject) {
        resolve()
      })
    },
    webOSWriteFile: function () {
      var self = this
      self.debug('webOSWriteFile')
    }, */
    tizenWriteFile: function (path, filename, blob) {
      var self = this
      return new Promise(function (resolve, reject) {
        tizen.filesystem.resolve(
          path + '/' + filename,
          function (file) {
            self.debug('tizenWriteFile - file exists - ' + filename)
            reject()
          },
          function (e) {
            // self.debug('tizenWriteFile - file dont exists - ' + filename)
            tizen.filesystem.resolve(
              self.tizenFolder,
              function (dir) {
                var newFile = dir.createFile(filename);
                // self.debug('newFile - ' + newFile)
                if (newFile != null) {
                  newFile.openStream(
                    'w',
                    function (fs) {
                      fs.write(blob)
                      fs.close()
                      // self.debug('Saved! - ')
                      resolve(filename)
                    },
                    function (e) {
                      self.debug('Error - ' + e.message)
                      reject()
                    },
                    'UTF-8'
                  )
                }
              },
              function (e) {
                self.debug('Error - ' + e.message)
                reject()
              }
            )
          }
        )
      })
    },
    tizenFolderFiles: function (path) {
      var self = this
      return new Promise(function (resolve, reject) {
        tizen.filesystem.resolve(
          self.tizenFolder,
          function (dir) {
            dir.listFiles(
              function (files) {
                var final = []
                for (var i = 0; i < files.length; i++) {
                  var fname = files[i].name
                  if (fname.indexOf('.__mira_completed') != -1) {
                    var fileName = fname.replace('.__mira_completed', '')
                    final.push(fileName)
                  }
                }
                resolve(final)
              },
              function (e) {
                reject()
              }
            )
          },
          function (e) {
            reject()
          }
        )
      })
    },
    tizenFolderFilesAll: function (path) {
      var self = this
      return new Promise(function (resolve, reject) {
        tizen.filesystem.resolve(
          self.tizenFolder,
          function (dir) {
            dir.listFiles(
              function (files) {
                resolve(files)
              },
              function (e) {
                reject()
              }
            )
          },
          function (e) {
            reject()
          }
        )
      })
    },
    cordovaFolderFilesAll: function (path) {
      var self = this
      return new Promise(function (resolve, reject) {
        window.resolveLocalFileSystemURL(self.dataFolder + path, function (fs) {
          var reader = fs.createReader()
          reader.readEntries(function (files) {
            var final = []
            _.forEach(files, function (el, idx) {
              if (el.isFile) {
                final.push(el.name)
              }
            })
            resolve(final)
          }, reject)
        }, function (e) {
          // self.$q.notify({ message: 'cordovaFolderFiles ERROR2 - ' + e.message, color: 'red-10' })
          reject()
        })
      })
    },
    cordovaFolderFiles: function (path) {
      var self = this
      return new Promise(function (resolve, reject) {
        window.resolveLocalFileSystemURL(self.dataFolder + path, function (fs) {
          var reader = fs.createReader()
          reader.readEntries(function (files) {
            var final = []
            _.forEach(files, function (el, idx) {
              if (el.isFile && el.name.indexOf('.__mira_completed') != -1) {
                var fileName = el.name.replace('.__mira_completed', '')
                final.push(fileName)
              }
            })
            /* each element has this structure:
            {
              isFile: true,
              isDirectory: false,
              name: 'abcdefgh.mp4',
              nativeURL: 'file:///storage/emulated/0/pictures/miravision_player/abcdefgh.mp4',
              fullPath: '/pictures/miravision_player/abcdefgh.mp4',
            }
            */
            resolve(final)
          }, reject)
        }, function (e) {
          // self.$q.notify({ message: 'cordovaFolderFiles ERROR2 - ' + e.message, color: 'red-10' })
          reject()
        })
      })
    },
    downloadFile: function (el) {
      var self = this
      if (self.downloadingFile != '') return
      if (self.parent.hasInternet == false) return
      if (self.filesExistInLocalstorage[el.id]) return // make sure we dont download twice

      self.downloadingFile = el.filename
      self.debug('Will download - ' + self.area.configKey + '#' + self.area.areaKey + ' - ' + el.filename)
      // console.log(el.url)
      fetch(el.url)
        .then(function (res) {
          return res.blob()
        })
        .then(function (blob) {
          // console.log(el.id)
          // console.log(blob)
          if (blob.type != 'text/html') {
            self.debug('download complete - ' + el.filename)
            self.dbContents.setItem(el.id, blob).then(function (blob) {
              self.filesExistInLocalstorage[el.id] = 1
              var imageURI = window.URL.createObjectURL(blob)
              self.$set(self.parent.slideshowFilesInCache, el.id, imageURI)
              self.downloadingFile = ''
              setTimeout(function () {
                self.buildMediasAvailable(true)
                self.checkAndDownloadFiles()
              }, 1000)
            }).catch(function (err) {
              self.debug('Failed storing in database... - ' + err.message, 'red')
              delete self.parent.slideshowFilesInCache[el.id]
              self.downloadingFile = ''
              self.checkAndDownloadFiles()
            })
          } else {
            self.parent.debug('download error1 - ' + el.filename, 'red')
            delete self.parent.slideshowFilesInCache[el.id]
            self.downloadingFile = ''
            self.checkAndDownloadFiles()
          }
        })
        .catch(function (error) {
          self.parent.debug('download error2 - ' + el.filename + ' - ' + error, 'red')
          delete self.parent.slideshowFilesInCache[el.id]
          self.downloadingFile = ''
          self.checkAndDownloadFiles()
        })
    },
    startPlaying: function () {
      var self = this
      // console.log('startPlaying', self.area.config)

      self.loops = 0
      self.slide = 0

      // sync with widgets param needs to be activated
      if (self.syncWithWidgets * 1 == 1) {
        /* var keyToUse = self.device_id + '#' + self.area.configKey + '#' + self.area.areaKey.replace('area', '') + '#cur_slide'
        localStorage.setItem(keyToUse, self.slide) */
        // send to iframes (widgets) info about the slide that is being played
        self.parent.wsPublishToWidgets({
          origin: 'browser',
          listen_ip: self.parent.kioskIp,
          websocket_token: self.parent.kioskToken,
          device_id: self.device_id,
          configKey: self.area.configKey,
          areaKey: self.area.areaKey,
          action: 'params',
          params: {
            to_player_uuid: self.parent.playerUniqueUuid,
            config_key: self.area.configKey,
            area_key: self.area.areaKey,
            slide_to_show: self.slide
          }
        })
      }

      // auto-start is disabled
      var dontAutostart = self.area.config.dont_autostart || 0;
      if (dontAutostart) return

      if (self.mediasAvailable.length == 0) return
      var elem = self.mediasAvailable[self.slide]

      if (elem) {
        if (elem.type == 'image') {
          var img1 = self.$refs[self.elemImg]
          if (img1) {
            img1.style.display = 'block'
            img1.src = elem['local_url']
            self.contentBeingPlayed = elem['local_url']
            img1.onload = function () {
              self.handleObjectFit(this.naturalWidth, this.naturalHeight)
            }
            // console.log('img1', img1.naturalWidth, img1.naturalHeight)
            var timeout = self.customTimeouts.hasOwnProperty(self.slide) ? self.customTimeouts[self.slide] : self.timeout
            clearTimeout(self.timeoutStart)
            self.timeoutStart = setTimeout(function () {
              self.contentOnEnded()
            }, timeout)
          }
        } else if (elem.type == 'video') {
          var vid1 = self.$refs[self.elemVideo]
          vid1.style.display = 'block'
          vid1.src = elem['local_url']
          self.contentBeingPlayed = elem['local_url']
          vid1.muted = self.muted
          vid1.play()
        } else if (elem.type == 'audio') {
          // audio contents
          self.playAudioElem(elem)
          self.contentBeingPlayed = elem['local_url']
        }
      }
    },
    playAudioElem: function (elem) {
      var self = this
      // if we change emission, avoid playing audio again
      var audio1 = self.$refs[self.elemAudio]
      audio1.src = elem.local_url
      audio1.id = elem.id
      // audio1.src = self.parent.http_method + self.parent.apiUrl + '/device_images/84C0EFD7BF6A/config_1/area3/slideshow/O00001_sample-9s.mp3'
      self.debug('playAudioElem - ' + elem.filename + ' - ' + elem.local_url)
      // audio1.play()
    },
    logContentBeingDisplayed: function (elem) {
      var self = this
      if (!self.parent.hasInternet) return
      var url = self.parent.http_method + self.parent.apiUrl + '/log_content_being_displayed'
      var xhr = new XMLHttpRequest()
      xhr.timeout = 10000
      xhr.onreadystatechange = function () {
        if (xhr.readyState == XMLHttpRequest.DONE) {
          if (xhr.status == 200) {
            // self.debug('log_content_being_displayed SUCCESS - ' + xhr.responseText)
          } else {
            // self.debug('log_content_being_displayed ERROR - ' + xhr.statusText)
          }
        }
      }
      xhr.open('POST', url)
      var data = new FormData()
      data.append('device_id', self.device_id)
      data.append('config', self.area.configKey)
      data.append('area', self.area.areaKey)
      data.append('file', elem.filename)
      xhr.send(data)
    },
    audioOnError: function (e) {
      var self = this
      // var elem = self.$refs[self.elemAudio]%
      var audio = e.srcElement
      self.debug('Audio error... ' + e.type + ' - ' + e.message + ' - ' + audio.id)
      /* self.dbContents.removeItem(audio.id)
      setTimeout(function () {
        location.reload()
      }, 3000) */
    },
    audioOnCanPlayThrough: function (e) {
      var self = this
      self.debug('audioOnCanPlayThrough')
      var audio = e.srcElement
      audio.play()
    },
    audioOnPlay: function (e) {
      var self = this
      self.debug('audioOnPlay')
      var media = self.mediasAvailable[self.slide]
      if (self.log_content_being_displayed == 1) self.logContentBeingDisplayed(media)
      if (!self.audiosDuration.hasOwnProperty(media.id)) {
        var now = new Date(self.parent.curDate)
        self.$set(self.audiosWhenStarted, media.id, now)
      }
      // self.debug('audioOnPlay - end')
    },
    audioOnMetaData: function (e) {
      var self = this
      self.debug('audioOnMetaData')
      // var elem = self.$refs[self.elemAudio]
      var audio = e.srcElement
      if (self.muted) audio.muted = self.muted
    },
    contentOnEnded: function (videoPlayError) {
      var self = this
      // self.debug('contentOnEnded!!')
      // console.log('contentOnEnded!!!', self.slide)

      if (!videoPlayError) videoPlayError = false

      var vid1 = self.$refs[self.elemVideo]
      var img1 = self.$refs[self.elemImg]
      if (!vid1 || !img1) return

      var curElem = self.mediasAvailable[self.slide]

      // calculate video duration, if not done yet, if prev video gave an error
      // we won't calc duration or sync stats
      if (curElem && !videoPlayError) {
        if (curElem.type == 'video') {
          if (!self.videosDuration.hasOwnProperty(curElem.id) && self.videosWhenStarted.hasOwnProperty(curElem.id)) {
            var now = new Date(self.parent.curDate)
            var secsPlayed = parseInt((now - self.videosWhenStarted[curElem.id]) / 1000)
            self.$set(self.videosDuration, curElem.id, secsPlayed)
            // self.debug('VIDEO DURATION - ' + secsPlayed + ' - ' + curElem.id)
          }
        } else if (curElem.type == 'audio') {
          if (!self.audiosDuration.hasOwnProperty(curElem.id) && self.audiosWhenStarted.hasOwnProperty(curElem.id)) {
            var now = new Date(self.parent.curDate)
            var secsPlayed = parseInt((now - self.audiosWhenStarted[curElem.id]) / 1000)
            self.$set(self.audiosDuration, curElem.id, secsPlayed)
            // self.debug('AUDIO DURATION - ' + secsPlayed + ' - ' + curElem.id)
          }
        }

        // collect stats of the content played
        if (self.contentId) {
          self.dbStats.getItem('stats').then(function (stats) {
            var curStats = []
            if (stats) curStats = stats
            // means the array is growing and there's no network connection to
            // sync stats... clean up the array to protect the device
            if (curStats.length > 2000) curStats = []
            // self.debug('CUR STATS - ' + curStats.length)
            var now = new Date(self.parent.curDate)
            var id = curElem.id + '_' + dateFormat(now, 'yyyymmddHHMMss')
            var duration = self.timeout / 1000
            if (curElem.type == 'video') {
              duration = self.videosDuration[curElem.id]
            } else if (curElem.type == 'audio') {
              duration = self.audiosDuration[curElem.id]
            }
            curStats.push({
              type: curElem.type,
              file: curElem.filename,
              secs_played: duration,
              content_id: self.contentId,
              screen: self.area.configKey,
              area: self.area.areaKey,
              area_type: 'slideshow',
              created: dateFormat(now, 'yyyy-mm-dd HH:MM:ss')
            })
            self.dbStats.setItem('stats', curStats).then(function (stats) { })
          })
        }
      }

      var prevIndex = self.slide
      var index = self.slide + 1

      if (self.dont_forward_emission * 1 == 1 || self.downloadingFile || self.parent.playlists.length == 1) {
        // shuffle contents if needed
        if (self.shuffle && self.mediasAvailable.length > 1) self.shuffle_index++

        if (index >= self.mediasAvailable.length) {
          self.slide = 0
        } else {
          self.slide = index
        }

      } else if (self.parent.playlists.length > 1) {
        if (index >= self.mediasAvailable.length) {
          // we've more than one playlist, we're not downloading contents and we dont block forwards
          // console.log('WILL EMIT playlist-next - ', self.device_id + '#' + self.area.configKey)
          self.$emit('playlist-next', self.device_id + '#' + self.area.configKey)
          return
        } else {
          self.slide = index
        }
      }

      /* if (self.area.areaKey == 'area2') {
        console.log('slide: ', self.slide)
      } */

      if (self.keepInSyncTag != '') {
        if (self.mediasAvailable.length > 1 && self.slide == 0) {
          // self.parent.debug('passei no fim!!!!')
          self.$emit('slideshow-sync', self.keepInSyncTag)
        }
      }

      // console.log('contentOnEnded - will play slide ', self.slide)
      var elem = self.mediasAvailable[self.slide]

      // sync with widgets param needs to be activated
      if (self.syncWithWidgets * 1 == 1) {
        /* var keyToUse = self.device_id + '#' + self.area.configKey + '#' + self.area.areaKey.replace('area', '') + '#cur_slide'
        localStorage.setItem(keyToUse, self.slide) */
        // send to iframes (widgets) info about the slide that is being played
        self.parent.wsPublishToWidgets({
          origin: 'browser',
          listen_ip: self.parent.kioskIp,
          websocket_token: self.parent.kioskToken,
          device_id: self.device_id,
          configKey: self.area.configKey,
          areaKey: self.area.areaKey,
          action: 'params',
          params: {
            to_player_uuid: self.parent.playerUniqueUuid,
            config_key: self.area.configKey,
            area_key: self.area.areaKey,
            slide_to_show: self.slide
          }
        })
      }

      if (elem) {
        if (elem.type == 'image') {
          vid1.style.display = 'none'
          img1.style.display = 'block'
          img1.src = elem['local_url']
          self.contentBeingPlayed = elem['local_url']
          img1.onload = function () {
            self.handleObjectFit(this.naturalWidth, this.naturalHeight)
          }
          var timeout = self.customTimeouts.hasOwnProperty(self.slide) ? self.customTimeouts[self.slide] : self.timeout
          clearTimeout(self.timeoutStart)
          if (self.log_content_being_displayed == 1) self.logContentBeingDisplayed(elem)
          self.timeoutStart = setTimeout(function () {
            self.contentOnEnded()
          }, timeout)
        } else if (elem.type == 'video') {
          img1.style.display = 'none'
          vid1.style.display = 'block'
          // if we've a single video, don't define src again to avoid flickering
          // if (self.mediasAvailable.length > 1) vid1.src = elem['local_url']
          vid1.src = elem['local_url']
          self.contentBeingPlayed = elem['local_url']
          vid1.muted = self.muted
          vid1.play()
        } else if (elem.type == 'audio') {
          // audio contents
          img1.style.display = 'none'
          vid1.style.display = 'none'
          self.playAudioElem(elem)
          self.contentBeingPlayed = elem['local_url']
        }
      }
    },
    videoOnError: function (e) {
      var self = this
      var elem = self.mediasAvailable[self.slide]

      self.$q.notify({
        message: 'Video error, type: ' + e.type + ', slide: ' + self.slide + ', file: ' + elem.filename,
        color: 'red-10'
      })

      self.$set(self.contentsToIgnore, elem.id, 1)
      self.buildMediasAvailable(true)

      if (self.parent.isTizen()) {
        // self.parent.debug('file delete - video error - ' + elem.id)
        self.tizenFileDelete(self.tizenFolder, elem.id).then(function () {
        })
        self.tizenFileDelete(self.tizenFolder, elem.id + '.__mira_completed').then(function () {
        })
      } else if (self.parent.isAndroid()) {
        // self.parent.debug('file delete - video error - ' + elem.id)
        self.cordovaFileDelete(self.androidFolder, elem.id).then(function () {
        })
        self.cordovaFileDelete(self.androidFolder, elem.id + '.__mira_completed').then(function () {
        })
      } else if (!self.parent.isLocalhost) {
        // self.parent.debug('file delete - video error - ' + elem.id)
        self.dbContents.removeItem(elem.id).then(function () {
        })
      }

      self.contentOnEnded(true)
    },
    videoOnPlay: function (e) {
      var self = this
      // self.debug('videoOnPlay - begin')
      var media = self.mediasAvailable[self.slide]
      if (self.log_content_being_displayed == 1) self.logContentBeingDisplayed(media)
      if (!self.videosDuration.hasOwnProperty(media.id)) {
        var now = new Date(self.parent.curDate)
        self.$set(self.videosWhenStarted, media.id, now)
      }
      // self.debug('videoOnPlay - end')
    },
    videoOnMetaData: function (e) {
      var self = this
      var elem = self.$refs[self.elemVideo]
      if (elem) self.handleObjectFit(elem.videoWidth, elem.videoHeight)
      // putting the muted on the loadmetadata event was the only way to make
      // it work in tizen 4.0
      var vid = e.srcElement
      vid.muted = self.muted
    }
  },
  watch: {
    'downloadingFile': function (val) {
      var self = this
      if (val) {
        // a download started
        self.downloadingCount++
      } else {
        // a download completed
        self.downloadingCount--
        self.buildMediasAvailable(true)

        self.downloadingDebug = Object.keys(self.parent.slideshowFilesInCache).length + ' / ' + self.mediasAvailableImagesFirst.length

        // when we're using a sync tag for syncrnonized start, we need to reload
        // when all downloads finish so all devices start ok
        if (self.keepInSyncTag != '') {
          setTimeout(function () {
            if (self.mediasAvailable.length == self.areaFiles.length) {
              // self.parent.debug('FIM!!!')
              self.parent.restartSlideshows(self.keepInSyncTag)
              if (_.includes(self.parent.isTizen() && self.parent.tvFirmwareVersion, '2.4')) {
                setTimeout(function () {
                  location.reload()
                }, 1000)
              }
            }
          }, 1000)
        }
      }
      // console.log('self.downloadingCount!', self.downloadingCount)
    },
    'parent.setupGlobalPlaylistTrigger': function () {
      var self = this
      console.log('slideshow watch! parent.setupGlobalPlaylistTrigger')
      self.buildMediasAvailable(true, true)
    },
    'parent.playlistRestart': function (res) {
      var self = this
      self.setup()
    },
    'parent.slideshowSyncTrigger': function () {
      var self = this
      // ignore this if we don't have a sync tag or we're downloading contents
      if (self.keepInSyncTag == '' || self.downloadingFile != '') return
      // this sync isn't for us
      if (self.parent.slideshowSync != self.keepInSyncTag) return
      // console.log('aqui!!!')
      self.parent.slideshowSync = ''
      var valid = true
      // make sure we don't do tons of restarts
      if (self.lastSlideshowRestart) {
        var now = new Date(self.parent.curDate)
        var seconds = Math.abs(now.getTime() - self.lastSlideshowRestart.getTime()) / 1000
        // console.log('seconds!', seconds)
        if (seconds < 4) valid = false
      }
      if (valid) {
        self.lastSlideshowRestart = new Date(self.parent.curDate)
        // self.parent.debug('slideshowSyncTrigger!')
        var elem = self.mediasAvailable[self.slide]
        // if the actual element is a video, pause it before we restart
        if (elem != undefined && elem.type == 'video') {
          var vid1 = self.$refs[self.elemVideo]
          vid1.pause()
        }
        self.setup(true)
      }
    },
    'area': function (event) {
      var self = this
      // console.log('area change!!!!', event)
      self.areaWidth = self.area.width * self.factor_width
      self.areaHeight = self.area.height * self.factor_height
      // self.startArea()
    },
    'shuffle_index': function (value) {
      var self = this

      // console.log('watch shuffle_index', self.shuffle_index)
      // console.log('shuffle', self.shuffle)
      // console.log('shuffle_number', self.shuffle_number)
      // console.log('value', value)

      // means files are still downloading
      if (self.mediasAvailable.length != self.areaFiles.length) return

      if (self.mediasAvailable.length > 1) {
        if (self.shuffle_number == 0) {
          console.log('will shuffle...')
          self.mediasAvailable = _.shuffle(self.mediasAvailable)
          /* _.forEach(self.mediasAvailable, function (e) {
            console.log('-----', e.filename)
          }) */
        } else {
          if (value == self.shuffle_number) {
            console.log('will shuffle...')
            self.shuffle_index = 0
            self.mediasAvailable = _.shuffle(self.mediasAvailable)
            /* _.forEach(self.mediasAvailable, function (e) {
              console.log('-----', e.filename)
            }) */
          }
        }
      }
    },
  },
  template: '<div>' +
    '<div v-show="error" class="absolute-center">' +
    '  <q-spinner-ios color="primary" style="font-size: 13vh;"/>' +
    '</div>' +
    '<div @click="doClick" v-show="!error">' +
    '  <video :ref="elemVideo" ' +
    '    poster="images/video-poster.png"' +
    '    :style="\'outline: none; display: none; position: absolute; left: 0px; top: 0px; border: 0px; object-fit: \' + object_fit + \';\' + borderRadius"' +
    '    :width="areaWidth"' +
    '    :height="areaHeight"' +
    '    @ended="contentOnEnded()"' +
    '    @error="videoOnError"' +
    '    @play="videoOnPlay"' +
    '    @loadedmetadata="videoOnMetaData"' +
    '  ></video>' +
    '  <img :ref="elemImg" ' +
    '    :width="areaWidth"' +
    '    :height="areaHeight"' +
    '    :style="\'outline: none; display: none; position: absolute; left: 0px; top: 0px; border: 0px; object-fit: \' + object_fit + \';\' + borderRadius"' +
    '  >' +
    '  <audio :ref="elemAudio" ' +
    '    @ended="contentOnEnded()"' +
    '    @error="audioOnError"' +
    '    @play="audioOnPlay"' +
    '    @canplaythrough="audioOnCanPlayThrough"' +
    '    @loadedmetadata="audioOnMetaData"' +
    '  ></audio>' +
    '</div>'
})
// -----------------------------------------------------------------------
// END SLIDESHOW
// -----------------------------------------------------------------------
