// -----------------------------------------------------------------------
// COMPONENT - SOUND
// -----------------------------------------------------------------------
Vue.component('sound', {
  props: {
    area: {
      type: Object
    },
    parent: {
      type: Object
    },
    device_id: {
      type: String
    },
    factor_width: {
      type: Number
    },
    factor_height: {
      type: Number
    }
  },
  data: function () {
    return {
      mediaData: [],
      media: [],
      songs_to_play: [],
      shuffle_index: 0,
      shuffled_times: 0,
      shuffled_media: [],
      index: 0,
      source: undefined,
      gainNode: undefined,
      context: new (window.AudioContext || window.webkitAudioContext)(),
      playing_playlist_area: [],
      playing: [],
      media_obj: {},
      onended: { looping: true },
      stop_playing: false,
      dbStats: undefined,
      contentId: '',
      dont_forward_emission: 0,
      timeoutSyncStats: undefined,
      timerStats: undefined
    }
  },
  created: function () {
    var self = this
    if (self.parent.isLocalhost) {
      self.urlToUse = '127.0.0.1'
    } else {
      self.urlToUse = 'apis.opalaconsult.com'
    }
    self.dbStats = localforage.createInstance({
      // name: 'DB_Miravision_Stats_' + self.area.configKey + '_' + self.area.areaKey,
      name: 'DB_Miravision_AudioStats',
      version: 1.0,
      driver: [
        localforage.INDEXEDDB,
        localforage.LOCALSTORAGE,
        localforage.WEBSQL,
      ]
    })
    // we start the sync stats procedure 10 secs after start so we have time to
    // know if we've internet or not
    clearTimeout(self.timeoutSyncStats)
    self.timeoutSyncStats = setTimeout(function () {
      if (self.contentId) {
        self.syncStats()
        clearInterval(self.timerStats)
        self.timerStats = self.parent.createInterval(function () {
          self.syncStats()
        }, self.parent.getRandomInt(300, 600) * 1000)
        // console.log('####fábio####', self.timerStats)
        // }, self.parent.getRandomInt(300, 600) * 1000)
      }
    }, 10000)
  },
  mounted: function () {
    var self = this
    self.setup()
  },
  computed: {
    myVar: function () {
    }
  },
  methods: {
    setup: function () {
      var self = this
      // if (self.area.config.play_one_time) {
      //   self.onended.looping = false
      // } else {
      //   self.onended.looping = true
      // }
      if (!self.area.config.content_id) {
        self.area.config.content_id = ''
      } else {
        self.contentId = self.area.config.content_id
      }
      if (self.area.config.hasOwnProperty('dont_forward_emission')) self.dont_forward_emission = self.area.config.dont_forward_emission * 1
      // console.log('EMISSION: ' + self.dont_forward_emission)
      // console.log('####AREA####', self.area)
      // console.log('####config####', self.config)
      // console.log('####media_obj####', self.media_obj)
      self.getFiles()
    },
    debug: function (msg) {
      var self = this
      console.log(msg)
      self.$emit('debug', 'audio - ' + msg)
    },
    getFiles: function () {
      var self = this
      var url = self.parent.http_method + self.parent.apiUrl + '/get_area_files?device_id=' + self.device_id + '&config_key=' + self.area.configKey + '&area_key=' + self.area.areaKey + '&complement=/audio'
      // self.debug('getFiles - ' + url)
      var xhr = new XMLHttpRequest()
      xhr.open('GET', url, false)
      xhr.send()
      if (xhr.status == 200) {
        var ref = JSON.parse(xhr.responseText)
        if (!ref.error) {
          self.getAreaFilesRun(ref)
        } else {
          self.debug('get area files error')
        }
      }
    },
    getAreaFilesRun: function (filesArr) {
      var self = this
      var files = []
      _.forEach(filesArr['files'], function (file, key) {
        files.push(file)
      })
      var sorted = files.sort()
      self.mediaData = sorted
      self.media = self.processMediaFiles(sorted)
      self.addToPlaylist(self.area.areaKey)
      self.playSounds()
      // self.mediaElementsCreate()
      // if (self.area.config.play_on_start * 1 == 1) {
      //   self.addToPlaylist(self.area.areaKey)
      //   self.playSounds()
      // }
    },
    processMediaFiles: function (files) {
      var self = this
      var media = []
      // console.log('processMediaFiles', files)
      if (files && files.length) {
        for (var key in files) {
          var file = files[key]
          var mp3 = file.toLowerCase().indexOf('.mp3')
          if (mp3 != -1) {
            var url = self.parent.http_method + self.urlToUse + '/queues_backoffice/device_images/' + self.device_id + '/' + self.area.configKey + '/' + self.area.areaKey + '/audio/' + file
            media.push(url)
          } else {
            self.debug('file not mp3')
          }
        }
      }
      return media
    },
    playSounds: function () {
      var self = this
      // console.log('playSounds')

      for (var s in self.media) {
        var sound = self.media[s]
        self.songs_to_play.push(sound)
      }

      if (self.area.config.shuffle * 1 == 1) {
        console.log('WILL SHUFFLE!!!')
        var newArray = _.clone(self.songs_to_play)
        newArray = _.shuffle(newArray)
        var control = 0
        while (_.isEqual(self.songs_to_play, newArray) || newArray[0] == self.songs_to_play[self.songs_to_play.length - 1]) {
          console.log('  need reshuffle - ' + control)
          newArray = _.shuffle(self.songs_to_play)
          if (control++ == 30) break
        }
        self.songs_to_play = newArray
        self.shuffled_media = self.songs_to_play
        self.songs_to_play = self.shuffled_media
        self.shuffle_index = 0
        self.shuffled_times++
        self.loadSound(self.context, self.songs_to_play[0])
      }
      else if (
        self.area.config.shuffle * 1 == 1 &&
        self.shuffled_times > 0 &&
        JSON.stringify(self.shuffled_media) != JSON.stringify(self.songs_to_play)
      ) {
        self.songs_to_play = self.shuffled_media
        self.loadSound(self.context, self.songs_to_play[0])
      } else {
        self.loadSound(self.context, self.songs_to_play[0])
      }
    },
    loadSound: function (context, soundUrl) {
      var self = this
      if (!soundUrl) return
      var arr = soundUrl.split('/')
      if (self.area.config.log_content_being_displayed) {
        // self.debug('loadSound - ' + arr[arr.length - 1])
      }
      var request = new XMLHttpRequest()
      request.open('GET', soundUrl, true)
      request.responseType = 'arraybuffer'
      request.onload = function () {
        if (!request.response || request.response.byteLength < 500) {
          setTimeout(function () {
            console.log('will try to play again...')
            self.loadSound(context, soundUrl)
          }, 500)
        } else {
          context.decodeAudioData(request.response, function (buffer) {
          self.playSound(context, buffer, soundUrl)
          })
        }
      }
      request.send()
    },
    playSound: function (context, buffer, soundUrl) {
      var self = this
      var started = new Date(self.parent.curDate)
      var file
      // log audio being played
      var arr = soundUrl.split('/')
      if (arr.length > 0) {
        file = arr[arr.length - 1]
        if (self.area.config.log_content_being_displayed) {
          setTimeout(function () {
            self.logContentBeingDisplayed(file)
          }, 1000)
        }
      }
      self.source = context.createBufferSource()
      self.source.onended = function () {
        var now = new Date(self.parent.curDate)
        var secsPlayed = parseInt((now - started) / 1000)
        // console.log('ONENDED', secsPlayed)
        self.dbStats.getItem('stats').then(function(stats) {
          var curStats = []
          if (stats) curStats = stats
          // means the array is growing and there's no network connection to
          // sync stats... clean up the array to protect the device
          if (curStats.length > 2000) curStats = []
          // self.debug('CUR STATS - ' + curStats.length)
          var now = new Date(self.parent.curDate)
          // var id = file + '_' + dateFormat(now, 'yyyymmddHHMMss')
          // var duration = self.timeout / 1000
          // if (curElem.type == 'video') duration = self.videosDuration[curElem.id]
          curStats.push({
            type: 'audio',
            file: file,
            secs_played: secsPlayed,
            content_id: self.contentId,
            screen: self.area.configKey,
            area: self.area.areaKey,
            area_type: 'audio',
            created: dateFormat(now, 'yyyy-mm-dd HH:MM:ss')
          })
          self.dbStats.setItem('stats', curStats).then(function(stats) {})
          // console.log('CURSTATS', curStats)
        })
        self.index++
        if (self.area.config.use_n_musics) {
          if (self.area.config.use_n_musics * 1 == self.index) {
            self.handleResume()
            return
          }
        }
        self.songs_to_play.shift()
        if (self.songs_to_play.length) {
          if (!self.stop_playing) self.loadSound(context, self.songs_to_play[0])
        } else {
          // self.still_playing = false
          //self.handleResume()
          if (self.area.config.shuffle * 1 == 1) self.shuffle_index++
          if (self.onended.params) {
            if (self.onended.params.onended) self.onended.params.onended = JSON.stringify(self.onended.params.onended)
            // console.log('SEND TO WS')
            // self.$emit('send-to-ws', self.onended.params)
          }
          if (self.onended.looping) {
            // console.log('EMIT')
            if (self.dont_forward_emission == 0) self.$emit('playlist-next', self.device_id + '#' + self.area.configKey)
            // self.playSounds()
          }
        }
      }
      self.source.buffer = buffer
      self.gainNode = self.context.createGain()
      self.gainNode.connect(self.context.destination)
      self.source.connect(self.gainNode)
      self.gainNode.gain.value = self.area.config.volume || 1
      self.source.start(0)
      // save played time for stats sync (we only sync contents that have a content_id defined)
      // if (self.area.config.content_id) {
      //   var arr = soundUrl.split('/')
      //   self.$dbInsert('stats_audio', {
      //     id: self.config_key + '#' + self.area_key + '#' + uid(),
      //     type: 'audio',
      //     file: arr[arr.length - 1],
      //     secs_played: Math.round(buffer.duration),
      //     created: date.formatDate(Date.now(), 'YYYY-MM-DD HH:mm:ss')
      //   })
      // }
    },
    syncStats: function () {
      var self = this
      // console.log('syncStats')
      if (!self.parent.hasInternet) {
        self.debug('cant sync stats because there is no Internet connection')
        return
      }
      self.dbStats.getItem('stats').then(function(stats) {
        var curStats = []
        if (stats) curStats = stats
        if (curStats.length == 0) return
        // clean the actual stats
        var url = self.parent.http_method + self.parent.apiUrl + '/stats_collect_form_method'
        var xhr = new XMLHttpRequest()
        xhr.onreadystatechange = function () {
          if (xhr.readyState == XMLHttpRequest.DONE) {
            if (xhr.status == 200) {
              self.dbStats.setItem('stats', []).then(function(stats) {})
              // self.debug('STATS SYNC SUCCESS - ' + xhr.responseText)
            } else {
              // self.debug('STATS SYNC ERROR - ' + xhr.statusText)
            }
          }
        }
        var toSend = {
          device_id: self.device_id,
          stats: curStats
        }
        xhr.open('POST', url)
        var data = new FormData()
        data.append('json', JSON.stringify(toSend))
        xhr.send(data)
      })
    },
    handleResume: function () {
      var self = this
      self.playing = ''
      self.media_obj = {
        play: [],
        pause: [],
        resume: self.playing_playlist_area
      }
      console.log('handleResume', self.media_obj)
    },
    addToPlaylist: function (data) {
      var self = this
      // console.log('DATA => ', data)
      if (self.playing_playlist_area.indexOf(data) == -1) self.playing_playlist_area.push(data)
        // console.log('PLAYLIST AREA => ', self.playing_playlist_area)
    },
    playAreaAudios: function (audios, onended) {
      var self = this
      console.log('playAreaAudios')
      self.stop_playing = true
      if (self.source) self.source.stop()
      self.source = undefined
      var timeout = setTimeout(function () {
        self.stop_playing = false
        if (onended) {
          var parsed = JSON.parse(onended)
          self.onended = parsed
        } else {
          self.onended = { looping: true }
        }
        self.songs_to_play = []
        self.media = []
        if (audios == 'playlist') {
          self.media = self.processMediaFiles(self.mediaData)
          // console.log(self.media)
          self.playSounds()
          clearTimeout(timeout)
          // console.log('PLAY SOUNDS => ', self.media)
        } else {
          var audioArray = audios.split(',')
          self.media = self.processMediaFiles(audioArray)
          self.playSounds()
          clearTimeout(timeout)
        }
      }, 1000)
    },
    playThisAudio: function (audio, onended) {
      var self = this
      console.log('playThisAudio')
      self.stop_playing = true
      self.source.stop()
      self.source = undefined
      var timeout = setTimeout(function () {
        self.stop_playing = false
        var parsed = JSON.parse(onended)
        self.onended = parsed
        self.songs_to_play = []
        self.media = []
        self.media.push(audio)
        self.playSounds()
        clearTimeout(timeout)
      }, 1000)
    },
    pause: function (onended) {
      var self = this
      console.log('pause')
      if (self.context.state == 'running') {
        self.stop_playing = true
        self.volumeDown()
        setTimeout(function () {
          self.context.suspend()
          if (onended) {
            var parsed = JSON.parse(onended)
            self.onended = parsed
          } else {
            self.onended = { looping: true }
          }
          if (self.onended.params) {
            var strOnended = JSON.stringify(self.onended.params.onended)
            self.$emit('send-to-ws', strOnended)
          }
        }, 500)
      }
    },
    play: function (onended) {
      var self = this
      if (self.context.state == 'suspended') {
        self.stop_playing = false
        self.context.resume()
        self.volumeUp()
        if (onended) {
          var parsed = JSON.parse(onended)
          self.onended = parsed
        } else {
          self.onended = { looping: true }
        }
      }
    },
    volumeUp: function () {
      var self = this
      if (!self.gainNode) return
      var timeout1 = setTimeout(function () {
        self.gainNode.gain.value = 0.1
        clearTimeout(timeout1)
      }, 50)
      if (self.area.config.volume == 0.1) return
      var timeout2 = setTimeout(function () {
        self.gainNode.gain.value = 0.2
        clearTimeout(timeout2)
      }, 100)
      if (self.area.config.volume == 0.2) return
      var timeout3 = setTimeout(function () {
        self.gainNode.gain.value = 0.3
        clearTimeout(timeout3)
      }, 150)
      if (self.area.config.volume == 0.3) return
      var timeout4 = setTimeout(function () {
        self.gainNode.gain.value = 0.4
        clearTimeout(timeout4)
      }, 200)
      if (self.area.config.volume == 0.4) return
      var timeout5 = setTimeout(function () {
        self.gainNode.gain.value = 0.5
        clearTimeout(timeout5)
      }, 250)
      if (self.area.config.volume == 0.5) return
      var timeout6 = setTimeout(function () {
        self.gainNode.gain.value = 0.6
        clearTimeout(timeout6)
      }, 300)
      if (self.area.config.volume == 0.6) return
      var timeout7 = setTimeout(function () {
        self.gainNode.gain.value = 0.7
        clearTimeout(timeout7)
      }, 350)
      if (self.area.config.volume == 0.7) return
      var timeout8 = setTimeout(function () {
        self.gainNode.gain.value = 0.8
        clearTimeout(timeout8)
      }, 400)
      if (self.area.config.volume == 0.8) return
      var timeout9 = setTimeout(function () {
        self.gainNode.gain.value = 0.9
        clearTimeout(timeout9)
      }, 450)
      if (self.area.config.volume == 0.9) return
      var timeout10 = setTimeout(function () {
        self.gainNode.gain.value = 1
        clearTimeout(timeout10)
      }, 500)
    },
    volumeDown: function () {
      var self = this
      if (!self.gainNode) return
      if (self.area.config.volume > 0.8) {
        var timeout1 = setTimeout(function () {
          self.gainNode.gain.value = 0.9
          clearTimeout(timeout1)
        }, 50)
      }
      if (self.area.config.volume > 0.7) {
        var timeout2 = setTimeout(function () {
          self.gainNode.gain.value = 0.8
          clearTimeout(timeout2)
        }, 100)
      }
      if (self.area.config.volume > 0.6) {
        var timeout3 = setTimeout(function () {
          self.gainNode.gain.value = 0.7
          clearTimeout(timeout3)
        }, 150)
      }
      if (self.area.config.volume > 0.5) {
        var timeout4 = setTimeout(function () {
          self.gainNode.gain.value = 0.6
          clearTimeout(timeout4)
        }, 200)
      }
      if (self.area.config.volume > 0.4) {
        var timeout5 = setTimeout(function () {
          self.gainNode.gain.value = 0.5
          clearTimeout(timeout5)
        }, 250)
      }
      if (self.area.config.volume > 0.3) {
        var timeout6 = setTimeout(function () {
          self.gainNode.gain.value = 0.4
          clearTimeout(timeout6)
        }, 300)
      }
      if (self.area.config.volume > 0.2) {
        var timeout7 = setTimeout(function () {
          self.gainNode.gain.value = 0.3
          clearTimeout(timeout7)
        }, 350)
      }
      if (self.area.config.volume > 0.1) {
        var timeout8 = setTimeout(function () {
          self.gainNode.gain.value = 0.2
          clearTimeout(timeout8)
        }, 400)
      }
      var timeout9 = setTimeout(function () {
        self.gainNode.gain.value = 0.1
        clearTimeout(timeout9)
      }, 450)
      var timeout10 = setTimeout(function () {
        self.gainNode.gain.value = 0
        clearTimeout(timeout10)
      }, 500)
    },
    logContentBeingDisplayed: function (file) {
      var self = this
      // console.log('logContentBeingDisplayed', file)
      var url = self.parent.http_method + self.parent.apiUrl + '/log_content_being_displayed'
      var data = new FormData()
      data.append('device_id', self.device_id)
      data.append('config', self.area.configKey)
      data.append('area', self.area.areaKey)
      data.append('file', file)


      var xhr = new XMLHttpRequest()
      xhr.open('POST', url, true)
      xhr.onload = function () {
          // do something to response
          // console.log(xhr.responseText)
      }
      xhr.send(data)
    },
  },
  watch: {
    'parent.playlistRestart': function (res) {
      var self = this
      self.setup()
    },
    'media_obj': function (value) {
      var self = this
      if (value.play.indexOf(self.area.areaKey) != -1) {
        console.log('PLAY => ', self.area.areaKey)
        self.playAreaAudios('playlist', '')
      }
      if (value.pause.indexOf(self.area.areaKey) != -1) {
        console.log('PAUSE => ', self.area.areaKey)
        self.pause()
      }
      if (value.resume.indexOf(self.area.areaKey) != -1) {
        console.log('RESUME => ', self.area.areaKey)
        self.play()
      }
    }
  },
  template: '<div>' +
  '</div>'
})
// -----------------------------------------------------------------------
// END SOUND
// -----------------------------------------------------------------------
