// -----------------------------------------------------------------------
// COMPONENT - TEXT
// -----------------------------------------------------------------------
Vue.component('text-area', {
  props: {
    area: {
      type: Object
    },
    parent: {
      type: Object
    },
    device_id: {
      type: String
    },
    factor_width: {
      type: Number
    },
    factor_height: {
      type: Number
    }
  },
  data: function () {
    return {
      text: '',
      alignCenter: false,
      borderRadius: ''
    }
  },
  created: function () {
    var self = this
    // self.parent.debug('text-area.js - created')
  },
  mounted: function () {
    var self = this
    self.setup()
  },
  methods: {
    setup: function () {
      var self = this
      // self.parent.debug('text-area.js - mounted')
      var cfg = self.area.config
      if (cfg.text) {
        var fontTranslation = {
          "'andale mono', monospace": 'andaleMono',
          "arial, helvetica, sans-serif": 'arial',
          "'arial black', sans-serif": 'arialBlack',
          "'book antiqua', palatino, serif": 'bookAntiqua',
          "'comic sans ms', sans-serif": 'comicSans',
          "'courier new', courier, monospace": 'courier',
          "georgia, palatino, serif": 'georgia',
          "helvetica, arial, sans-serif": 'helvetica',
          "impact, sans-serif": 'impact',
          "tahoma, arial, helvetica, sans-serif": 'tahoma',
          "terminal, monaco, monospace": 'terminal',
          "'times new roman', times, serif": 'times',
          "'trebuchet ms', geneva, sans-serif": 'trebuchet',
          "verdana, geneva, sans-serif": 'verdana',
          "webdings": 'webdings',
          "wingdings, 'zapf dingbats'": 'wingding'
        }
        _.forEach(fontTranslation, function (el, key) {
          cfg.text = cfg.text.split(key).join(el)
        })
        cfg.text = cfg.text.split('<strong>').join('<b>')
        cfg.text = cfg.text.split('</strong>').join('</b>')
        self.text = cfg.text
        // console.log('self.text', self.text)
      }
      if (cfg.text_align == 1) self.alignCenter = true
    }
  },
  watch: {
    'parent.playlistRestart': function (res) {
      var self = this
      self.setup()
    },
    'parent.wsConnected': function (res) {
      var self = this
      // console.log('text-area - wsConnected')
    },
    'parent.wsMessage': function (res) {
      var self = this
      var action = res.action || ''
      if (! action) {
        console.error('Missing action parameter...')
        return
      }
      switch (action) {
        case 'text_update':
          var area = res.area || ''
          var text = res.text || ''
          if (! area) {
            console.error('Missing area parameter...')
            return
          }
          if ('area' + area != self.area.areaKey) return
          if (! text) {
            console.error('Missing text parameter...')
            return
          }
          console.log('text_update', text)
          // node.textContent = text
          var elem = self.$refs.text
          elem.innerHTML = text
          self.text = text
          break
        default:
          // console.log('invalid action - ', res.action)
      }
    }
  },
  template: '<div>' +
  '<div ref="text" v-html="text" :class="alignCenter ? \'absolute-center\' : \'\'" style="width: 100%;"></div>' +
  '</div>'
})
// -----------------------------------------------------------------------
// END TEXT
// -----------------------------------------------------------------------
