// -----------------------------------------------------------------------
// COMPONENT - WEBPAGE
// -----------------------------------------------------------------------
Vue.component('web-page', {
  props: {
    area: {
      type: Object
    },
    parent: {
      type: Object
    },
    device_id: {
      type: String
    },
    factor_width: {
      type: Number
    },
    factor_height: {
      type: Number
    }
  },
  data: function () {
    return {
      scrollPosition: 0,
      scrollSpeed: 1,
      reloadSecs: 0,
      timerReload: undefined
    }
  },
  created: function () {
    var self = this
  },
  mounted: function () {
    var self = this
    self.setup()
  },
  computed: {
    width: function () {
      var self = this
      return (self.area.width * self.factor_width) + 'px'
    },
    height: function () {
      var self = this
      return (self.area.height * self.factor_height) + 'px'
    }
  },
  methods: {
    setup: function () {
      var self = this

      self.reloadSecs = self.area.config.reload_in_secs * 1 || 0

      if (self.reloadSecs > 0) {
        // reload
        clearInterval(self.timerReload)
        self.timerReload = setInterval(function () {
          var iframe = self.$refs.myiframe
          if (iframe && iframe.contentWindow) {
            // console.log('will reload...')
            iframe.contentWindow.location.reload()
          }
        }, self.reloadSecs * 1000)
      }

      self.setPage()

      /* setTimeout(function () {
        console.log('will scroll')
        // self.scrollIframe()
      }, 5000) */

      /* if (self.reloadAfter > 0) {
        setInterval(function () {
          console.log('reload!')
          self.setPage()
        }, self.reloadAfter * 1000)
      } */
    },
    // it's not in use because cross origin control blocks the possibility to
    // make it scroll
    scrollIframe: function () {
      var self = this

      // Increment the scroll position
      self.scrollPosition += self.scrollSpeed

      var iframe = self.$refs.myiframe
      // console.log('iframe', iframe)

      // Scroll the iframe
      iframe.contentWindow.scrollTo(0, self.scrollPosition)

      // Check if we have reached the bottom of the iframe
      if (self.scrollPosition >= iframe.contentDocument.documentElement.scrollHeight - iframe.clientHeight) {
          // Reset the scroll position to the top
          self.scrollPosition = 0
      }

      // Request the next frame for smooth scrolling
      requestAnimationFrame(self.scrollIframe)

      // Increment the scroll position
      self.scrollPosition += self.scrollSpeed

      // Scroll the iframe
      iframe.contentWindow.scrollTo(0, self.scrollPosition)

      // Check if we have reached the bottom of the iframe
      if (self.scrollPosition >= iframe.contentDocument.documentElement.scrollHeight - iframe.clientHeight) {
          // Reset the scroll position to the top
          self.scrollPosition = 0
      }

      // Request the next frame for smooth scrolling
      requestAnimationFrame(self.scrollIframe)
    },
    setPage: function () {
      var self = this
      var useProxy = self.area.config.webpage_proxy * 1 || 0
      var openInNewBrowser = self.area.config.open_in_new_browser * 1 || 0

      // we use this option in android only
      if (!self.parent.isAndroid()) {
        openInNewBrowser = 0
      }

      // this will open a dedicated browser for the url. it was created because
      // we caught some url with html that's incompatible with the android webview
      // but be loaded in an external browser. I've found Fullscreen Browser 4.27
      // to be perfect for it.
      if (openInNewBrowser == 1) {
        var ref = cordova.InAppBrowser.open(self.area.config.url, '_system')
      } else {
        var page = ''

        if (useProxy == 1) {
          page = 'http://webproxy.opalaconsult.com:55400/?url=' + self.area.config.url
        } else {
          page = self.area.config.url
        }

        return page
      }
    }
  },
  watch: {
    'parent.playlistRestart': function (res) {
      var self = this
      self.setup()
    }
  },
  template: '<div :style="\'width:\' + width + \'; height:\' + height + \';\'">' +
  '<iframe :src="setPage()" ref="myiframe" frameborder="0" scrolling="no" style="width: 100%; height: 100%;"></iframe>' +
  '</div>'
})
// -----------------------------------------------------------------------
// END WEBPAGE
// -----------------------------------------------------------------------
