// -----------------------------------------------------------------------
// COMPONENT - WIDGET
// -----------------------------------------------------------------------
Vue.component('widget',
  {
  props: {
    area: {
      type: Object
    },
    parent: {
      type: Object
    },
    device_id: {
      type: String
    },
    factor_width: {
      type: Number
    },
    factor_height: {
      type: Number
    },
    kiosk_ip: {
      type: String
    },
    kiosk_token: {
      type: String
    },
    orientation: {
      type: String
    }
  },
  data: function () {
    return {
      comp_id: Math.random(),
      time: '',
      triedLoadRemote: false,
      didLoad: false,
      ignoreIframeContentCheck: false
    }
  },
  created: function () {
    var self = this
    self.parent.createdWidgetIds.push(self.area.config.widget_id)
  },
  mounted: function () {
    var self = this
    self.setup()
  },
  computed: {
    width: function () {
      var self = this
      return (self.area.width * self.factor_width) + 'px'
    },
    height: function () {
      var self = this
      return (self.area.height * self.factor_height) + 'px'
    },
    userUuid: function () {
      var self = this
      var userUuid = self.area.config.user_uuid
      if (self.parent.curPlaylistConfig.hasOwnProperty('user_uuid_widget')) {
        if (self.parent.curPlaylistConfig['user_uuid_widget']) {
          userUuid = self.parent.curPlaylistConfig['user_uuid_widget']
        }
      }
      return userUuid
    }
  },
  methods: {
    setup: function () {
      var self = this
      // self.parent.debug('widget setup()')
      // console.log('widget setup()')
      // console.log('AREA', self.area)
      // console.log('IP', self.kiosk_ip)
      // console.log('orientation', self.orientation)

      // in preview mode, allways load from the internet
      if (self.parent.queryParam('from_backoffice') == 1) {
        self.playFromInternet(5)
        return;
      }

      var iframe = document.getElementById(self.comp_id)

      // this URL starts with 'widgets/...' to try and load widget from local player
      // if fails, will load it from the internet after
      var urlToCheck = self.buildWidgetUrl() + '?' + self.buildWidgetParamsString()

      setTimeout(function () {
        var iframe = document.getElementById(self.comp_id)
        iframe.onload = function() {
          // after we figure out that we don't have a local widget, we'll load it
          // from the internet, for that reason we have this flag and we return
          // when we're loading an internet widget
          if (self.ignoreIframeContentCheck) return
          // Content loaded successfully
          var iframeContent = iframe.contentDocument || iframe.contentWindow.document
          if (iframeContent && iframeContent.body) {
            if (iframeContent.body.innerHTML == '') {
              // self.parent.debug('No content from widget...', 'red')
              self.playFromInternet(1)
            } else {
              //
              // IMPORTANT!!!
              // all local widgets need to have a q-app id to be considered valid
              //
              if (!iframeContent.getElementById("q-app")) {
                self.playFromInternet(4)
              } else {
                // self.parent.debug('WIDGET - ' + self.area.config.widget.value + ' - playFromLocal')
              }
            }
          } else {
            // self.parent.debug('Error getting iframe content...', 'red')
            self.playFromInternet(2)
          }
        }
        iframe.onerror = function() {
          // Error loading content
          // self.parent.debug('Error loading content', 'red')
          self.playFromInternet(3)
        }
        iframe.onloadend = function() {
          self.parent.debug('onloadend')
        }
        // self.parent.debug('setup() urlToCheck - ' + urlToCheck)
        // tries to load widget from player itself, if fails, tries from internet
        // iframe.setAttribute('src', urlToCheck)
        self.playFromInternet(6)
      }, 500)
    },
    playFromInternet: function (idx) {
      var self = this
      self.ignoreIframeContentCheck = true
      // self.parent.debug('WIDGET - ' + self.area.config.widget.value + ' - playFromInternet - ' + idx)
      var url_player = self.parent.http_method + self.parent.apiUrl
      url_player = url_player + '/' + self.buildWidgetUrl() + '?' + self.buildWidgetParamsString()
      // self.parent.debug(url_player)
      var iframe = document.getElementById(self.comp_id)
      iframe.setAttribute('src', url_player)
    },
    debug: function (msg) {
      var self = this
      console.log(msg)
      self.$emit('debug', 'widget - ' + msg)
    },
    buildWidgetUrl: function () {
      var self = this
      var url
      if (self.area.config.widget.type == 'user') {
        url = 'widgets/users/' + self.userUuid + '/' + self.area.config.widget.value + '/index.html'
      } else if  (self.area.config.widget.type == 'brand') {
        url = 'widgets/brands/' + self.area.config.widget.brand_id + '/' + self.area.config.widget.value + '/index.html'
      } else {
        url = 'widgets/global/' + self.area.config.widget.value + '/index.html'
      }
      return url
    },
    buildWidgetParamsString: function () {
      var self = this
      var storeFoId = self.parent.storeFoId || ''
      var auxDeviceId = self.parent.auxDeviceId || ''

      var options = {
        year: 'numeric',
        month: 'numeric',
        day: 'numeric',
        hour: 'numeric',
        minute: 'numeric',
        second: 'numeric',
      }

      var url_player = self.parent.http_method + self.parent.apiUrl
      // console.log('url_player', url_player)

      // '&url=' + url_player +
      var paramsString = 'ip_screen=' + self.kiosk_ip +
        '&device_id=' + self.device_id +
        '&widget_id=' + self.area.config.widget_id +
        '&width=' + self.width +
        '&height=' + self.height +
        '&random=' + Math.random() +
        '&player_uuid=' + self.parent.playerUniqueUuid +
        '&time=' + self.parent.curDate.toLocaleDateString('pt-PT', options) +
        '&websocket_token=' + self.kiosk_token +
        '&config_key=' + self.area.configKey +
        '&area_key=' + self.area.areaKey +
        '&factor_width=' + self.factor_width +
        '&factor_height=' + self.factor_height +
        '&orientation=' + self.orientation +
        '&type=' + self.area.config.widget.type +
        '&store_fo_id=' + storeFoId +
        '&aux_device_id=' + auxDeviceId +
        '&url=' + url_player

      return paramsString
    }
  },
  watch: {
    'parent.playlistRestart': function (res) {
      var self = this
      // console.log('parent.playlistRestart!!!!')
      self.setup()
    }
  },
  template:
  '<div :style="\'width:\' + width + \'; height:\' + height + \';\'">' +
    '<iframe :id="comp_id" width="100%" height="100%" frameborder="0" scrolling="no"></iframe>' +
  '</div>'
})
// -----------------------------------------------------------------------
// END WIDGET
// -----------------------------------------------------------------------
